# TODO implement tests for this!
def get_reader_from_filename(filename):
    """
    Returns an instance of an implementation of :py:class:`pypore.i_o.abstract_reader.AbstractReader` based on the
    extension of filename.

    :param string filename: Filename to get the reader for.
    :returns: An open reader of an implementation of :py:class:`pypore.i_o.abstract_reader.AbstractReader` based on the\
             extension of filename, for the following extensions.

        - '.h5' extension: returns an instance of :py:class:`pypore.i_o.data_file_reader.DataFileReader`.
        - '.log' extension: returns :py:class:`pypore.i_o.chimera_reader.ChimeraReader`.

        If the file extension is not in the above list, None is returned.

    :raise: Exception - this method opens a reader, which could throw an exception.

    >>> reader = get_reader_from_filename('test.h5') # Returns an open DataFileReader
    >>> data = reader.get_all_data()
    """

    # TODO implement for other file types

    if filename[-len('.h5'):] == '.h5':
        from data_file_reader import DataFileReader as ReaderClass
    elif filename[-len('.log'):] == '.log':
        from chimera_reader import ChimeraReader as ReaderClass
    elif filename[-len('.hkd'):] == '.hkd':
        from heka_reader import HekaReader as ReaderClass
    elif filename[-len('.hex'):] == '.hex':
        from cnp2_reader import CNP2Reader as ReaderClass
    else:
        raise ValueError(
            "No default match for the extension of {0}. Default extensions include '.h5', '.log', '.hkd', '.hex'.".format(filename))

    reader = ReaderClass(filename)
    return reader

    # elif '.hkd' in filename:
    #     return open_heka_file(filename, decimate)
    # elif '.mat' in filename:
    #     return open_gabys_file(filename, decimate)
    #
    # return {'error': 'File not specified with correct extension. Possibilities are: \'.log\', \'.hkd\''}



import unittest
import numpy as np
from pyqtgraph.graphicsItems.PlotCurveItem import PlotCurveItem
from pyporegui.graphicsItems.my_plot_item import MyPlotItem
from pyporegui.graphicsItems.path_item import PathItem


class TestMyPlotItem(unittest.TestCase):

    def test_my_plot_item_add_item(self):
        plot_item = MyPlotItem()
        x = [1]
        y = [1]
        plot = PlotCurveItem(x, y)
        plot_item.addItem(plot)

        self.assertEqual(len(plot_item._myItemList), 1)

        x = y = np.zeros(2)
        plot2 = PathItem(x, y)
        plot_item.addItem(plot2)
        self.assertEqual(len(plot_item._myItemList), 2)

    def test_add_event_item(self):
        plot_item = MyPlotItem()
        x = [1]
        y = [1]
        plot = PlotCurveItem(x, y)
        plot_item.add_event_item(plot)

        self.assertEqual(len(plot_item._myEventItemList), 1)

        x = y = np.zeros(2)
        plot2 = PathItem(x, y)
        plot_item.add_event_item(plot2)
        self.assertEqual(len(plot_item._myEventItemList), 2)

    def test_clear_event_items(self):
        plot_item = MyPlotItem()
        x = [1]
        y = [1]
        plot = PlotCurveItem(x, y)
        plot_item.addItem(plot)

        plot2 = PlotCurveItem(y, x)
        plot_item.add_event_item(plot2)

        self.assertEqual(len(plot_item._myEventItemList), 1)
        self.assertEqual(len(plot_item.listDataItems()), 2)

        plot_item.clear_event_items()
        self.assertEqual(len(plot_item._myEventItemList), 0)

        self.assertEqual(len(plot_item.listDataItems()), 1)

    def test_clear(self):
        plot_item = MyPlotItem()
        x = [1]
        y = [1]
        plot = PlotCurveItem(x, y)
        plot_item.addItem(plot)

        plot2 = PlotCurveItem(y, x)
        plot_item.add_event_item(plot2)

        x = y = np.zeros(2)
        plot3 = PathItem(x, y)
        plot_item.add_event_item(plot3)

        plot_item.clear()
        self.assertEqual(len(plot_item.listDataItems()), 0)
        self.assertEqual(len(plot_item._myEventItemList), 0)
        self.assertEqual(len(plot_item._myItemList), 0)

    def test_plot_clear(self):
        # tests to make sure that when the clear flag is passed
        # to plots, that clear works as expected
        plot_item = MyPlotItem()
        x = [1]
        y = [1]
        plot = PlotCurveItem(x, y)
        plot_item.addItem(plot)

        plot2 = PlotCurveItem(y, x)
        plot_item.add_event_item(plot2)

        plot5 = PlotCurveItem(y, x)
        plot_item.addItem(plot5)

        x = y = np.zeros(2)
        plot3 = PathItem(x, y)
        plot_item.add_event_item(plot3)

        plot4 = plot_item.plot(clear=True)

        # Test that all of the plots have been cleared,
        # and that plot4 was added successfully
        self.assertEqual(len(plot_item.listDataItems()), 1)
        self.assertEqual(len(plot_item._myEventItemList), 0)
        self.assertEqual(len(plot_item._myItemList), 1)
        self.assertEqual(plot_item.listDataItems()[0], plot4)
        self.assertEqual(plot_item._myItemList[0], plot4)


import unittest

from pypore.strategies.absolute_change_threshold_strategy import AbsoluteChangeThresholdStrategy


class TestAbsoluteChangeThresholdStrategy(unittest.TestCase):
    def test_compute_starting_threshold(self):
        """
        Test that the starting and ending thresholds do not depend on the baseline or variance.
        """
        start_changes = [-1.e9, -1, -1.e-9, 0., 1, 1.e-9, 1.e9]
        end_changes = [-1.e9, -1, -1.e-9, 0., 1, 1.e-9, 1.e9]
        baselines = [-1.e9, -1., -1.e-9, 0., 1, 1.e-9, 1.e9]
        variances = [-1.e9, -1., -1.e-9, 0., 1, 1.e-9, 1.e9]
        for i in xrange(len(start_changes)):
            strategy = AbsoluteChangeThresholdStrategy(start_changes[i], end_changes[i])

            for j in xrange(len(variances)):
                start_thresh = strategy.compute_starting_threshold(baselines[j], variances[j])
                end_thresh = strategy.compute_ending_threshold(baselines[j], variances[j])

                self.assertEqual(start_thresh, start_changes[i],
                                 "Unexpected starting threshold. Should be {0}, was {1}.".format(start_changes[i],
                                                                                                 start_thresh))
                self.assertEqual(end_thresh, end_changes[i],
                                 "Unexpected ending threshold. Should be {0}, was {1}.".format(end_changes[i],
                                                                                               end_thresh))




from PySide import QtGui
from pyporegui._thread_manager import _ThreadManager
from pyporegui.file_items import FilterListItem, FileListItem
from pyporegui.widgets.event_analysis_plot_widget import EventAnalysisPlotWidget

__all__ = ['EventAnalysisTab']


class EventAnalysisTab(_ThreadManager, QtGui.QSplitter):
    """
    """

    def __init__(self, parent=None):
        """
        :param PySide.QtGui.QMainWindow parent: Parent window of this tab (optional).
        """
        super(EventAnalysisTab, self).__init__(parent)
        self._parent = parent

        options = self._create_event_analysis_options()
        self.event_analysis_plot_widget = EventAnalysisPlotWidget()

        # Put everything in baseline_filter_parameter scroll area
        scroll_options = QtGui.QScrollArea()
        scroll_plots = QtGui.QScrollArea()
        scroll_options.setWidgetResizable(True)
        scroll_plots.setWidgetResizable(True)

        scroll_options.setWidget(options)
        scroll_plots.setWidget(self.event_analysis_plot_widget)

        self.addWidget(scroll_options)
        self.addWidget(scroll_plots)

    def color_picker_btn_clicked(self):
        col = QtGui.QColorDialog.getColor(initial=self.event_color)

        if col.isValid():
            self.event_color = col
            self.frm.setStyleSheet("QWidget { background-color: %s }"
                                   % col.name())

    def add_filter_clicked(self):
        items = self.list_event_widget.selectedItems()
        if items is None or len(items) < 1:
            return

        params = self._get_current_event_analysis_params()

        file_names = []

        for item in items:
            file_names.append(item.get_file_name())

        item = FilterListItem(file_names, **params)
        self.list_filter_widget.addItem(item)

        self.event_analysis_plot_widget.add_selections(file_names, params)

    def _get_current_event_analysis_params(self):
        params = {'color': self.event_color}
        return params

    def open_event_databases(self, file_names=None):
        """
        Adds the files to the list widget.

        :param ListType<StringType> file_names: (Optional) List of file names to be added to the list widget. If not
                                                included, then a QtGui.QFileDialog will be opened to select files.
        """
        if file_names is None:
            file_names = QtGui.QFileDialog.getOpenFileNames(self, 'Open event database', '.', '*.h5')[0]

        if len(file_names) > 0:
            self.list_event_widget.clear()
        else:
            return
        for w in file_names:
            item = FileListItem(w)
            self.list_event_widget.addItem(item)

    def remove_filter_clicked(self):
        items = self.list_filter_widget.selectedItems()
        for item in items:
            index = self.list_filter_widget.indexFromItem(item).row()
            self.event_analysis_plot_widget.remove_filter(index)
            self.list_filter_widget.takeItem(index)

    def _on_event_file_selection_changed(self):
        """
        Called when the user clicks a new file in the file list.
        """
        self.btn_add_filter.setEnabled(True)

    def _create_event_analysis_options(self):
        """
        Initializes the analysis options on the left side of the tab.
        """
        scroll_area = QtGui.QScrollArea()
        scroll_area.setWidgetResizable(True)

        # Create baseline_filter_parameter list for files want to analyze
        self.list_event_widget = QtGui.QListWidget()
        self.list_event_widget.setMaximumHeight(100)
        self.list_event_widget.itemSelectionChanged.connect(self._on_event_file_selection_changed)
        self.list_event_widget.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        files_options = QtGui.QFormLayout()
        files_options.addRow('Event Databases:', self.list_event_widget)

        # # Color Picker
        self.event_color = QtGui.QColor('blue')
        pick_color_btn = QtGui.QPushButton()
        pick_color_btn.setText('Choose a Color')
        pick_color_btn.clicked.connect(self.color_picker_btn_clicked)

        self.frm = QtGui.QFrame()
        self.frm.setStyleSheet("QWidget { background-color: %s }"
                               % self.event_color.name())
        self.frm.setMinimumSize(15, 15)
        self.frm.setMaximumSize(30, 30)

        files_options.addRow(pick_color_btn, self.frm)

        # # List of filters created
        self.btn_add_filter = QtGui.QPushButton('Add selections as filter')
        self.btn_add_filter.clicked.connect(self.add_filter_clicked)
        self.btn_add_filter.setEnabled(False)
        form_filter = QtGui.QFormLayout()
        form_filter.addRow('Filters:', self.btn_add_filter)
        self.list_filter_widget = QtGui.QListWidget()
        vbox = QtGui.QVBoxLayout()
        vbox.addLayout(form_filter)
        vbox.addWidget(self.list_filter_widget)
        btn_remove_filter = QtGui.QPushButton('Remove selected filters')
        btn_remove_filter.clicked.connect(self.remove_filter_clicked)
        vbox.addWidget(btn_remove_filter)

        vbox_left = QtGui.QVBoxLayout()
        vbox_left.addLayout(files_options)
        vbox_left.addLayout(vbox)

        vbox_left_widget = QtGui.QWidget()
        vbox_left_widget.setLayout(vbox_left)

        scroll_area.setWidget(vbox_left_widget)

        return scroll_area


"""
Created on Aug 20, 2013

@author: `@parkin`_
"""
from PySide import QtGui
import unittest
import os

from PySide.QtGui import QColor

# This class needs a QApplication instance
if QtGui.qApp is None:
    QtGui.QApplication([])

# from PySide.QtTest import QTest
from pyporegui.file_items import FilterListItem, FileListItem, DataFileListItem


class TestFileListItem(unittest.TestCase):
    
    def setUp(self):
        self.filename = os.path.abspath('hi.txt')
        self._set_item()
        
    def _set_item(self):
        self.item = FileListItem(self.filename)
    
    def tearDown(self):
        pass
    
    def test_file_list_item_simple_name(self):
        simple_name = self.item.get_simple_name()
        self.assertEqual(simple_name, 'hi.txt')
        
    def test_file_list_item_directory(self):
        directory = self.item.get_directory()
        self.assertEqual(directory, os.path.dirname(self.filename))
        
    def test_file_list_item_filename(self):
        check = self.item.get_file_name()
        self.assertEqual(check, self.filename)
        
        
class TestDataFileListItem(TestFileListItem):
    """
    Subclassing TestFileListItem will also run those tests too.
    """
     
    def setUp(self):
        self.params = {'hi': 9, 'bye': 19}
        TestFileListItem.setUp(self)
        
    def _set_item(self):
        self.item = DataFileListItem(self.filename, self.params)
        
    def test_data_file_list_item_params(self):
        params = self.item.get_params()
        self.assertEqual(params, self.params)
        
        # Test immutable
        params['qwertyuiop'] = 1
        self.assertNotIn('qwertyuiop', self.item.get_params())
        
    def test_data_file_list_item_get_param(self):
        self.assertEqual(self.item.get_param('hi'), 9)
        self.assertEqual(self.item.get_param('bye'), 19)
        self.assertIsNone(self.item.get_param('sdafkfasfweoaifeawfksfd'))
        
_instance = None
    
    
class TestFilterListItem(unittest.TestCase):
    
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_filter_list_item_simple_name(self):
        file_names = [os.path.abspath('hi.txt')]
        item = FilterListItem(file_names)
        simple_names = item.get_simple_names()
        self.assertEqual(len(simple_names), len(file_names))
        self.assertEqual(simple_names[0], 'hi.txt')
        self.assertEqual(item.get_simple_name_at(0), 'hi.txt')
        self.assertIsNone(item.get_simple_name_at(100))
        self.assertIsNone(item.get_simple_name_at(-12))
        
        # Test immutability
        simple_names.append('blahblahblahblahblah')
        self.assertEqual(len(item.get_simple_names()), len(file_names))
        
    def test_filter_list_item_file_names(self):
        file_names = [os.path.abspath(__file__), os.path.abspath('x.hi')]
        item = FilterListItem(file_names)
        names = item.get_file_names()
        self.assertEqual(names, file_names)
        self.assertEqual(item.get_file_name_at(1), file_names[1])
        self.assertIsNone(item.get_file_name_at(-1))
        
        # Test immutability
        names.append('blahblahblahblah')
        self.assertEqual(len(item.get_file_names()), len(file_names))
        
    def test_filter_list_item_default_color(self):
        file_names = [os.path.abspath('hi.txt')]
        item = FilterListItem(file_names)
        color = item.get_params()['color']
        self.assertTrue(type(color) is QColor)
        
    def test_filter_list_item_color(self):
        file_names = [os.path.abspath('hi.txt')]
        color = QColor.fromRgbF(1., 1., 1., 1.)
        item = FilterListItem(file_names, color=color)
        color_check = item.get_params()['color']
        self.assertEqual(color, color_check)
        
    def test_filter_list_item_params(self):
        file_names = [os.path.abspath('hi.txt')]
        item = FilterListItem(file_names)
        params = item.get_params()
        self.assertEqual(len(params.keys()), 1)
        self.assertIn('color', params)
        
        item = FilterListItem(file_names, x=12, y={'hi': 'bye'})
        params = item.get_params()
        self.assertEqual(len(params.keys()), 3)
        self.assertIn('color', params)
        self.assertIn('x', params)
        self.assertIn('hi', params['y'])
        
        # tests immutability
        params['qwertyuiop'] = 1
        self.assertNotIn('qwertyuiop', item.get_params())
        
    def test_filter_list_item_text(self):
        file_names = [os.path.abspath('hi.txt'), os.path.abspath('bye.py'), os.path.abspath('q')]
        item = FilterListItem(file_names)
        text = item.text()
        self.assertEqual(text, 'hi.txt, bye.py, q')
        
        file_names = []
        item = FilterListItem(file_names)
        text = item.text()
        self.assertEqual(text, 'item')
    
    def test_filter_list_item_has_icon(self):
        file_names = [os.path.abspath('hi.txt')]
        color = QColor.fromRgbF(.5, .6, .4, .34)
        item = FilterListItem(file_names, color=color)
        icon = item.icon()
        self.assertIsNotNone(icon)
        self.assertFalse(icon.isNull())

if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



from PySide import QtGui, QtCore
from numpy import linspace
import pyqtgraph as pg
from pyqtgraph.widgets.LayoutWidget import LayoutWidget

from pypore.event_finder import Parameters
from pypore.strategies.absolute_change_threshold_strategy import AbsoluteChangeThresholdStrategy
from pypore.strategies.adaptive_baseline_strategy import AdaptiveBaselineStrategy
from pypore.strategies.fixed_baseline_strategy import FixedBaselineStrategy
from pypore.strategies.noise_based_threshold_strategy import NoiseBasedThresholdStrategy
from pypore.strategies.percent_change_threshold_strategy import PercentChangeThresholdStrategy
from pyporegui.my_threads import AnalyzeDataThread, PlotThread
from pyporegui.graphicsItems.my_plot_item import MyPlotItem
from pyporegui.widgets.plot_tool_bar import PlotToolBar
from base_tabs import BaseQSplitterDataFile


__all__ = ['EventFindingTab']


# Note _ThreadManager must go first, otherwise its __init__ is never called
class EventFindingTab(BaseQSplitterDataFile):
    """
    A QtGui.QSplitter that contains event finding options on the left and a plot on the right.
    """

    def __init__(self, parent=None):
        """
        :param PySide.QtGui.QMainWindow parent: Parent main window (optional).
        """
        # Define the instance elements
        self.events = []  # holds the events from the most recent analysis run
        self.plot_widget = None
        self.p1 = None
        self.plot_tool_bar = None
        super(EventFindingTab, self).__init__(parent)

    def get_current_analysis_parameters(self):
        """
        Reads the current analysis parameters from the gui and returns the results in a dictionary. Returns
        dictionary entry 'error' with text of the error if one exists.

        :returns: DictType -- the event analysis parameters. These include:

            - 'error' - Text if there was an error

            OR

            - 'min_event_length'
            - 'max_event_length'
            - 'baseline_strategy'
            - 'baseline_filter_parameter'
            - 'baseline_current'

            etc...
        """

        threshold_type = baseline_type = None
        # Get Min_event length in microseconds
        try:
            min_event_length = float(self.min_event_length_edit.text())
        except ValueError:
            return {'error': 'Could not read float from Min Event Length text box.  Please fix.'}
        # Get Max Event Length in microseconds
        try:
            max_event_length = float(self.max_event_length_edit.text())
        except ValueError:
            return {'error': 'Could not read float from Max Event Length text box.  Please fix.'}
        if min_event_length >= max_event_length:
            return {'error': 'Min Event Length is greater than Max Event Length.  Please fix.'}

        baseline_type_str = str(self.baseline_type_combo.currentText())
        if baseline_type_str == 'Adaptive':
            try:
                baseline_filter_parameter = float(self.baseline_filter_parameter_edit.text())
            except ValueError:
                return {'error': 'Could not read float from Baseline Filter Parameter text box.  Please fix.'}
            try:
                variance_filter_parameter = float(self.variance_filter_parameter_edit.text())
            except ValueError:
                return {'error': 'Could not read float from Variance Filter Parameter text box.  Please fix.'}
            baseline_type = AdaptiveBaselineStrategy(baseline_filter_parameter=baseline_filter_parameter,
                                                     variance_filter_parameter=variance_filter_parameter)
        elif baseline_type_str == 'Fixed':
            try:
                baseline_current = float(self.baseline_current_edit.text())
                baseline_type = FixedBaselineStrategy(baseline=baseline_current)
            except ValueError:
                return {'error': 'Could not read float from Baseline Current text box.  Please fix.'}

        threshold_direction_str = str(self.threshold_direction_combo.currentText())
        detect_positive_events = (threshold_direction_str in ('Positive', 'Both'))
        detect_negative_events = (threshold_direction_str in ('Negative', 'Both'))

        threshold_type_str = str(self.threshold_type_combo.currentText())
        if threshold_type_str == 'Noise Based':
            try:
                start_stddev = float(self.threshold_stdev_start.text())
            except ValueError:
                return {'error': 'Could not read float from Start StdDev text box.  Please fix.'}
            try:
                end_stddev = float(self.threshold_stdev_end.text())
            except ValueError:
                return {'error': 'Could not read float from End StdDev text box.  Please fix.'}
            threshold_type = NoiseBasedThresholdStrategy(start_std_dev=start_stddev, end_std_dev=end_stddev)
        elif threshold_type_str == 'Absolute Change':
            try:
                absolute_change_start = float(self.absolute_change_start_edit.text())
            except ValueError:
                return {'error': 'Could not read float from Absolute Change Start.  Please fix.'}
            try:
                absolute_change_end = float(self.absolute_change_end_edit.text())
            except ValueError:
                return {'error': 'Could not read float from Absolute Change End.  Please fix.'}
            threshold_type = AbsoluteChangeThresholdStrategy(change_start=absolute_change_start,
                                                             change_end=absolute_change_end)
        elif threshold_type_str == 'Percent Change':
            try:
                percent_change_start = float(self.percentage_change_start_edit.text())
            except ValueError:
                return {'error': 'Could not read float from Percent Change Start text box.  Please fix.'}
            try:
                percent_change_end = float(self.percentage_change_end_edit.text())
            except ValueError:
                return {'error': 'Could not read float from Percent Change End text box.  Please fix.'}
            threshold_type = PercentChangeThresholdStrategy(percent_change_start=percent_change_start,
                                                            percent_change_end=percent_change_end)

        parameters = Parameters(min_event_length=min_event_length, max_event_length=max_event_length,
                                detect_positive_events=detect_positive_events,
                                detect_negative_events=detect_negative_events, baseline_strategy=baseline_type,
                                threshold_strategy=threshold_type)
        return parameters

    def plot_data(self, plot_options):
        """
        Plots waveform in datadict.
        :param DictType plot_options: Dictionary of plot options. Must contain:

        * Data at plot_options['data'][0]
        * sample_rate at plot_options['sample_rate']
        * plot_range at plot_options['plot_range']. This can be [start,stop], or
          'all' for 0:n.

        """
        # Read the first file, store data in dictionary
        data = plot_options['data'][0]
        sample_rate = plot_options['sample_rate']
        plot_range = plot_options['plot_range']

        n = len(data)
        # If problem with input, just plot all the data
        if plot_range == 'all' or len(plot_range) != 2 or plot_range[1] <= plot_range[0]:
            plot_range = [0, n]
        else:  # no problems!
            n = plot_range[1] - plot_range[0] + 1

        ts = 1 / sample_rate

        times = linspace(ts * plot_range[0], ts * plot_range[1], n)
        y_data = data[plot_range[0]:(plot_range[1] + 1)]

        self.plot_widget.clear_event_items()
        self.p1.setData(x=times, y=y_data)
        self.plot_widget.autoRange()
        self._dispatch_process_events()

    def _analyze_data_thread_callback(self, results):
        """
        Callback for updates from the AnalyzeDataThread.
        """
        if 'status_text' in results:
            self._dispatch_status_update(results['status_text'])
        if 'Events' in results:
            single_plot = False
            events = results['Events']
            if len(events) < 1:
                return
            elif len(self.events) < 1:
                # if this is our first time plotting events, include the single event plot!
                single_plot = True
            self.events += events
            self.event_display_edit.setMaxLength(int(len(self.events) / 10) + 1)
            self.event_display_edit.setValidator(QtGui.QIntValidator(1, len(self.events), self.event_display_edit))
            self.eventCountText.setText('/' + str(len(self.events)))
            if self.plot_tool_bar.is_plot_during_checked():
                self.plotEventsOnMainPlot(events)
                self.addEventsToConcatEventPlot(events)
            if single_plot:
                self.event_display_edit.setText('1')
            self._dispatch_process_events()
            self.analyzethread.readyForEvents = True
        if 'done' in results:
            if results['done']:
                self.stop_analyze_button.setEnabled(False)

    def _on_analyze(self):
        """
        Searches for events in the file that is currently highlighted in the files list.
        """
        selected_items = self.file_list_widget.selectedItems()
        if len(selected_items) > 0:
            curr_items = selected_items
        else:
            return

        parameters = self.get_current_analysis_parameters()
        if isinstance(parameters, dict) and 'error' in parameters:
            self._dispatch_status_update(parameters['error'])
            return

        debug = self.debug_check_box.isChecked()

        # Clear the current events
        del self.events[:]
        # self.prev_concat_time = 0.

        file_names = [str(x.get_file_name()) for x in curr_items]

        self._dispatch_status_update("Event Count: 0 Percent Done: 0")

        # Start analyzing data in new analyzethread.
        self.analyzethread = AnalyzeDataThread(file_names, parameters, debug)
        self.analyzethread.dataReady.connect(self._analyze_data_thread_callback)
        self.add_thread(self.analyzethread)
        self.analyzethread.start()

        self.stop_analyze_button.setEnabled(True)

    def _on_analyze_stop(self):
        """
        Called when the user clicks the Stop button.
        """
        self.clean_threads()
        self.stop_analyze_button.setEnabled(False)
        self._dispatch_status_update("Analyze aborted.")

    def _on_file_item_doubleclick(self, item):
        """
        Called when baseline_filter_parameter file list item is double clicked.
        Starts the plotting thread, which opens the file, parses data, then passes to plotData
        """
        # adding by emitting signal in different thread
        self._dispatch_status_update("Plotting...")
        decimates = self.plot_tool_bar.is_decimate_checked()
        thread = PlotThread(self.p1, filename=str(item.get_file_name()), decimate=decimates)
        thread.dataReady.connect(self._on_file_item_doubleclick_callback)
        self.add_thread(thread)
        thread.start()

    def _on_file_item_doubleclick_callback(self, results):
        if 'plot_options' in results:
            self.plot_data(results['plot_options'])
        if 'status_text' in results:
            self._dispatch_status_update(results['status_text'])
        if 'thread' in results:
            self.remove_thread(results['thread'])

    def _on_file_item_selection_changed(self):
        self.analyze_button.setEnabled(True)

    def _create_right_widget(self, parent=None):
        wig = pg.GraphicsLayoutWidget(parent)

        # Main plot
        self.plot_widget = MyPlotItem(title='Current Trace', name='Plot')
        wig.addItem(self.plot_widget)
        self.plot_widget.enableAutoRange('xy', False)
        self.p1 = self.plot_widget.plot()  # create an empty plot curve to be filled later

        # Tool bar for main plot.  Contains zoom button and different checkboxes
        self.plot_tool_bar = PlotToolBar(self)
        if not parent is None:
            parent.addToolBar(self.plot_tool_bar)

        event_finder_plots_layout = LayoutWidget()
        event_finder_plots_layout.addWidget(self.plot_tool_bar, row=1, col=0, colspan=3)
        event_finder_plots_layout.addWidget(wig, row=2, col=0, colspan=3)

        return event_finder_plots_layout

    def open_data_files(self, file_names=None):
        are_files_opened = super(EventFindingTab, self).open_data_files(file_names)
        if are_files_opened:
            self.analyze_button.setEnabled(False)

    def _create_left_widget(self, parent=None):
        """
        Initializes everything in the options pane on the left side of the splitter.
        """
        scroll_area = QtGui.QScrollArea()
        scroll_area.setWidgetResizable(True)

        list_form_layout = super(EventFindingTab, self)._create_left_widget(parent)
        self.file_list_widget.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        # Other GUI controls
        #
        self.analyze_button = QtGui.QPushButton("&Find Events")
        self.connect(self.analyze_button, QtCore.SIGNAL('clicked()'), self._on_analyze)
        self.analyze_button.setEnabled(False)

        self.stop_analyze_button = QtGui.QPushButton("&Stop")
        self.connect(self.stop_analyze_button, QtCore.SIGNAL('clicked()'), self._on_analyze_stop)
        self.stop_analyze_button.setEnabled(False)

        self.debug_check_box = QtGui.QCheckBox()
        self.debug_check_box.setChecked(False)
        self.debug_check_box.setText('Debug')
        self.debug_check_box.setToolTip("When checked, extra data will be saved to the EventDatabase. This"
                                        " data includes the data, baseline, and thresholds at each datapoint.")

        # Analysis options
        self.min_event_length_edit = QtGui.QLineEdit()
        self.min_event_length_edit.setText('10.0')
        self.min_event_length_edit.setValidator(QtGui.QDoubleValidator(0, 1e12, 5, self.min_event_length_edit))
        self.max_event_length_edit = QtGui.QLineEdit()
        self.max_event_length_edit.setText('1000.0')
        self.max_event_length_edit.setValidator(QtGui.QDoubleValidator(0, 1e12, 5, self.max_event_length_edit))

        fixed_analysis_options = QtGui.QFormLayout()
        fixed_analysis_options.addRow('Min Event Length [us]:', self.min_event_length_edit)
        fixed_analysis_options.addRow('Max Event Length [us]:', self.max_event_length_edit)

        # Baseline options
        baseline_options = QtGui.QStackedLayout()
        self.baseline_type_combo = QtGui.QComboBox()
        self.baseline_type_combo.addItems(('Adaptive', 'Fixed'))
        self.baseline_type_combo.activated.connect(baseline_options.setCurrentIndex)

        adaptive_options_layout = QtGui.QFormLayout()
        self.baseline_filter_parameter_edit = QtGui.QLineEdit()
        self.baseline_filter_parameter_edit.setValidator(QtGui.QDoubleValidator(0, 10, 5, self.baseline_filter_parameter_edit))
        self.baseline_filter_parameter_edit.setText('0.93')
        adaptive_options_layout.addRow('Baseline Filter Parameter:', self.baseline_filter_parameter_edit)
        self.variance_filter_parameter_edit = QtGui.QLineEdit()
        self.variance_filter_parameter_edit.setValidator(QtGui.QDoubleValidator(0, 10, 5, self.variance_filter_parameter_edit))
        self.variance_filter_parameter_edit.setText('0.99')
        adaptive_options_layout.addRow('Variance Filter Parameter:', self.variance_filter_parameter_edit)
        # need to cast to widget to add to QStackedLayout
        adaptive_options_widget = QtGui.QWidget()
        adaptive_options_widget.setLayout(adaptive_options_layout)

        choose_baseline_btn = QtGui.QPushButton('Baseline:')
        choose_baseline_btn.setToolTip('Click to choose the baseline from the plot.')

        fixed_options_layout = QtGui.QFormLayout()
        self.baseline_current_edit = QtGui.QLineEdit()
        self.baseline_current_edit.setValidator(QtGui.QDoubleValidator(-9999, 9999, 9, self.baseline_current_edit))
        self.baseline_current_edit.setText('0.0')
        fixed_options_layout.addRow(choose_baseline_btn, self.baseline_current_edit)
        fixed_options_widget = QtGui.QWidget()
        fixed_options_widget.setLayout(fixed_options_layout)

        baseline_options.addWidget(adaptive_options_widget)
        baseline_options.addWidget(fixed_options_widget)

        baseline_form = QtGui.QFormLayout()
        baseline_form.addRow('Baseline Type:', self.baseline_type_combo)

        # Threshold options
        threshold_options = QtGui.QStackedLayout()
        self.threshold_type_combo = QtGui.QComboBox()
        self.threshold_type_combo.addItem('Noise Based')
        self.threshold_type_combo.addItem('Absolute Change')
        self.threshold_type_combo.addItem('Percent Change')
        self.threshold_type_combo.activated.connect(threshold_options.setCurrentIndex)

        threshold_form = QtGui.QFormLayout()
        self.threshold_direction_combo = QtGui.QComboBox()
        self.threshold_direction_combo.addItems(('Both', 'Positive', 'Negative'))
        threshold_form.addRow('Threshold Direction:', self.threshold_direction_combo)
        threshold_form.addRow('Threshold Type:', self.threshold_type_combo)

        noise_based_options_layout = QtGui.QFormLayout()
        self.threshold_stdev_start = QtGui.QLineEdit()
        self.threshold_stdev_start.setValidator(QtGui.QDoubleValidator(-9999, 9999, 4, self.threshold_stdev_start))
        self.threshold_stdev_start.setText('5.0')
        noise_based_options_layout.addRow('Start StdDev:', self.threshold_stdev_start)
        self.threshold_stdev_end = QtGui.QLineEdit()
        self.threshold_stdev_end.setValidator(QtGui.QDoubleValidator(-9999, 9999, 4, self.threshold_stdev_end))
        self.threshold_stdev_end.setText('1.0')
        noise_based_options_layout.addRow('End StdDev:', self.threshold_stdev_end)

        absolute_drop_options_layout = QtGui.QFormLayout()
        self.absolute_change_start_edit = QtGui.QLineEdit()
        self.absolute_change_start_edit.setValidator(
            QtGui.QDoubleValidator(-9999, 9999, 9, self.absolute_change_start_edit))
        self.absolute_change_start_edit.setText('0.1')
        absolute_drop_options_layout.addRow('Absolute Change Start [uA]:', self.absolute_change_start_edit)
        self.absolute_change_end_edit = QtGui.QLineEdit()
        self.absolute_change_end_edit.setValidator(
            QtGui.QDoubleValidator(-9999, 9999, 9, self.absolute_change_end_edit))
        self.absolute_change_end_edit.setText('0.0')
        absolute_drop_options_layout.addRow('Absolute Change End [uA]:', self.absolute_change_end_edit)

        percentage_change_options_layout = QtGui.QFormLayout()
        self.percentage_change_start_edit = QtGui.QLineEdit()
        self.percentage_change_start_edit.setValidator(
            QtGui.QDoubleValidator(0, 9999, 5, self.percentage_change_start_edit))
        self.percentage_change_start_edit.setText('10.0')
        percentage_change_options_layout.addRow('Percent Change Start:', self.percentage_change_start_edit)
        self.percentage_change_end_edit = QtGui.QLineEdit()
        self.percentage_change_end_edit.setValidator(
            QtGui.QDoubleValidator(0, 9999, 5, self.percentage_change_end_edit))
        self.percentage_change_end_edit.setText('0.0')
        percentage_change_options_layout.addRow('Percent Change End:', self.percentage_change_end_edit)

        noise_based_options = QtGui.QWidget()
        noise_based_options.setLayout(noise_based_options_layout)

        absolute_drop_options = QtGui.QWidget()
        absolute_drop_options.setLayout(absolute_drop_options_layout)

        percentage_change_options_widget = QtGui.QWidget()
        percentage_change_options_widget.setLayout(percentage_change_options_layout)

        threshold_options.addWidget(noise_based_options)
        threshold_options.addWidget(absolute_drop_options)
        threshold_options.addWidget(percentage_change_options_widget)

        hbox = QtGui.QHBoxLayout()

        for w in [self.analyze_button, self.stop_analyze_button]:
            hbox.addWidget(w)
            hbox.setAlignment(w, QtCore.Qt.AlignVCenter)

        # Left vertical layout with settings
        vbox_left = QtGui.QVBoxLayout()
        vbox_left.addLayout(list_form_layout)
        vbox_left.addLayout(fixed_analysis_options)
        vbox_left.addLayout(baseline_form)
        vbox_left.addLayout(baseline_options)
        vbox_left.addLayout(threshold_form)
        vbox_left.addLayout(threshold_options)
        vbox_left.addWidget(self.debug_check_box)
        vbox_left.addLayout(hbox)

        vbox_left_widget = QtGui.QWidget()
        vbox_left_widget.setLayout(vbox_left)

        scroll_area.setWidget(vbox_left_widget)

        return scroll_area



# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '0.0.3'
version = '0.0.3'
full_version = '0.0.3.dev20160127183730'
git_revision = '81e54b3d0864ee539966d42a16bc8b45b3e1e319'
release = False

if not release:
    version = full_version



import numpy as np


class ReaderTests(object):
    """
    Other test classes for readers should inherit this class in addition to unittest.TestCase.

    Subclasses **must** use multi-inheritance and also inherit from :py:class:`unittest.TestCase`.

    The subclass **must** have a field 'reader_class' that is a subclass of
    :py:class:`AbstractReader <pypore.i_o.abstract_reader.AbstractReader>`. For example::

        self.reader_class = ChimeraReader

    The subclasses also **must** override the following methods:

    #. :py:func:`help_scaling`
    #. :py:func:`help_scaling_decimated`
    #. :py:func:`help_get_all_data_returns_to_beginning`

    """
    reader_class = None
    default_test_data_files = None

    def test_get_all_data_returns_to_beginning(self):
        """
        Tests that calling :py:func:`get_all_data <pypore.i_o.abstract_reader.AbstractReader.get_all_data>` returns
        all of the data, no matter if we've already used the reader for other things.
        """
        file_names = self.help_get_all_data_returns_to_beginning()

        for filename in file_names:
            reader = self.reader_class(filename)

            orig_data = reader.get_all_data()[0]

            # Test getting all data back to back works
            data2 = reader.get_all_data()[0]

            equal = np.array_equal(orig_data, data2)
            self.assertTrue(equal, "get_all_data returns different data the 2nd time. "
                                   "Original data: {0}, 2nd time: {1}.".format(orig_data, data2))

            reader.close()

            # Test getting all data after getting next blocks works.
            reader = self.reader_class(filename)

            b = reader.get_next_blocks()

            data3 = reader.get_all_data()[0]
            equal = np.array_equal(orig_data, data3)
            self.assertTrue(equal, "get_all_data returns different data after calling get_next_blocks. "
                                   "Original data: {0}, after gnb call: {1}.".format(orig_data, data3))

    def help_get_all_data_returns_to_beginning(self):
        """
        If the subclass does **not** set self.default_test_data_files to a list of test files, then
        this method should be overridden.

        :returns: List of file names to be tested.
        """
        if self.default_test_data_files is not None:
            return self.default_test_data_files
        else:
            raise NotImplementedError('Inheritors should override this method or set self.default_test_data_files'
                                      ' to a list of test data files.')

    def test_scaling(self):
        """
        Tests that the data is scaled correctly, from a known test file.
        """
        file_names, means_should_be, std_devs_should_be = self.help_scaling()

        for i, filename in enumerate(file_names):
            mean_should_be = means_should_be[i]
            std_dev_should_be = std_devs_should_be[i]

            reader = self.reader_class(filename)

            data = reader.get_all_data()[0]
            reader.close()

            mean = np.mean(data)

            mean_diff = abs((mean - mean_should_be) / mean_should_be)
            self.assertLessEqual(mean_diff, 0.05,
                                 "Data mean in '{0}' scaled wrong. "
                                 "Should be {1}, got {2}.".format(filename, mean_should_be, mean))

            std_dev = np.std(data)

            std_diff = abs((std_dev - std_dev_should_be) / std_dev_should_be)
            self.assertLessEqual(std_diff, 0.05,
                                 "Baseline in '{0}' scaled wrong. "
                                 "Should be {1}, got {2}.".format(filename, std_dev_should_be, std_dev))

    def help_scaling(self):
        """
        Subclasses should override this method and return the following, in the following order.
        :returns: The following, in order:

        #. list of file names of the test files.
        #. list of known means of the data in the test files to check against the reader_class.
        #. list of known standard deviations of the data to check against the reader_class.

        """
        raise NotImplementedError('Inheritors should override this method.')

    def test_scaling_decimated(self):
        """
        Tests that the decimated data is scaled the same as the un-decimated data.
        """
        file_names = self.help_scaling_decimated()

        for filename in file_names:
            reader = self.reader_class(filename)

            data = reader.get_all_data()[0]
            data_decimated = reader.get_all_data(decimate=True)[0]
            reader.close()

            mean = np.mean(data)
            std = np.std(data)

            mean_decimated = np.mean(data_decimated)

            # compare the means, using the same standard deviation
            np_sqrt = np.sqrt(1.0 / data.size + 1.0 / data_decimated.size)
            denominator = (std * np_sqrt)
            # z test, check that less than 2 standard deviation difference.
            z = abs(mean - mean_decimated) / denominator
            self.assertLessEqual(z, 2,
                                 "Decimated mean in '{0}' scaled wrong. "
                                 "Should be {1}, got {2}. Z statistic: {3}".format(filename, mean, mean_decimated,
                                                                                   z))

    def help_scaling_decimated(self):
        """
        Checks that the decimated and un-decimated
        data :py:func:`get_all_data <pypore.i_o.abstract_reader.AbstractReader.get_all_data>`.

        If the subclass does **not** set self.default_test_data_files to a list of test files, then
        this method should be overridden.

        :returns: list of file names for testing decimated scaling.
        """
        if self.default_test_data_files is not None:
            return self.default_test_data_files
        else:
            raise NotImplementedError('Inheritors should override this method or set self.default_test_data_files'
                                      ' to a list of test data files.')

    def help_decimated_shape(self):
        """
        Helper for :py:func:`test_decimated_shape`.

        Subclasses **must** override this method and return the following.

        :returns: list of file names for testing decimated shape.
        """
        if self.default_test_data_files is not None:
            return self.default_test_data_files
        else:
            raise NotImplementedError('Inheritors should override this method or set self.default_test_data_files'
                                      ' to a list of test data files.')

    def test_decimated_shape(self):
        """
        Tests that the decimated shape is correct.
        """

        file_names = self.help_decimated_shape()

        for filename in file_names:
            reader = self.reader_class(filename)

            decimated = reader.get_all_data(decimate=True)[0]

            block_size = reader.get_block_size()

            points_per_channel_total = reader.get_points_per_channel_total()

            decimated_length = decimated.size

            # Should have floor(ppct/block_size) + extra 2 points if spare block on the end.
            decimated_length_should_be = 2 * int(points_per_channel_total / block_size)
            if points_per_channel_total % block_size > 0:
                decimated_length_should_be += 2

            self.assertEqual(decimated_length_should_be, decimated_length,
                             "Decimate length incorrect. Should be {0}. Was {1}.".format(decimated_length_should_be,
                                                                                         decimated_length))



from pyqtgraph.graphicsItems.ScatterPlotItem import SpotItem


class SpotItem(SpotItem):
    """
    A pyqtgraph.graphicsItems.ScatterPlotItem.SpotItem with an extra field for the event_position.
    """

    event_position = 0

    def __init__(self, data, plot, position):
        super(SpotItem, self).__init__(data, plot)
        self.event_position = position


import os
import unittest

import pypore.sampledata.testing_files as tf

abs_path = os.path.abspath(__file__)
abs_dir = os.path.dirname(abs_path)

TEST_FILES_PATH = os.path.abspath(os.path.join(abs_dir, '..', 'testDataFiles'))


class TestGetAbsPath(unittest.TestCase):
    def test_correct_paths(self):
        """
        Tests that the returned files are correct.
        """
        full_names = tf.get_all_file_names(with_path=True)
        short_names = tf.get_all_file_names(with_path=False)

        for i, short_name in enumerate(short_names):
            abs_path = tf.get_abs_path(short_name)
            self.assertEqual(full_names[i], abs_path,
                             msg="Absolute path not correct.\nShould be '{0}'.\nWas '{1}'.".format(full_names[i],
                                                                                                   abs_path))

    def test_not_exists_raises(self):
        """
        Test that an exception is raised if the filename is not present.
        """
        self.assertRaises(tf.TestFileDoesntExistError, tf.get_abs_path, 'blahblahblahblah.blahblahblah')


class TestGetAllFileNames(unittest.TestCase):
    def test_with_path(self):
        """
        Tests that getting all of the file names returns the correct info.
        """
        file_names_should_be = []

        for (dir_path, dir_names, file_names) in os.walk(TEST_FILES_PATH):
            for i, filename in enumerate(file_names):
                file_names[i] = os.path.join(dir_path, filename)
            file_names_should_be.extend(file_names)

        file_names = tf.get_all_file_names()

        # Currently there are 13 files in the folder.
        self.assertTrue(len(file_names), 13)
        self.assertEqual(sorted(file_names), sorted(file_names_should_be),
                         msg="Returned incorrect file names.\nShould"
                             " be {0}.\nWas {1}.".format(file_names_should_be, file_names))

        # Test that the file_names returned include their paths
        # Check the file 'chimera_small' is there
        is_chimera_small = False
        for filename in file_names:
            self.assertTrue('pypore' in filename and 'sampledata' in filename and 'testDataFiles' in filename,
                            msg="Filename should include path."
                                " Was '{0}'.".format(filename))
            if 'chimera_small.log' in filename:
                is_chimera_small = True

        self.assertTrue(is_chimera_small, msg="File {0} was not found, but was expected.".format('chimera_small.log'))

        file_names2 = tf.get_all_file_names(with_path=True)
        # Check we can pass with_path as an argument, and that True is default.
        self.assertEqual(sorted(file_names), sorted(file_names2))

    def test_no_path(self):
        """
        Tests that we can get all of the file names
        """
        file_names_should_be = []

        for (dir_path, dir_names, file_names) in os.walk(TEST_FILES_PATH):
            file_names_should_be.extend(file_names)

        file_names = tf.get_all_file_names(with_path=False)

        # Currently there are 13 files in the folder.
        self.assertTrue(len(file_names), 13)
        self.assertEqual(sorted(file_names), sorted(file_names_should_be),
                         msg="Returned incorrect file names.\nShould"
                             " be {0}.\nWas {1}.".format(file_names_should_be, file_names))

        # Test that the file_names returned do NOT include their paths
        # Check the file 'chimera_small' is there
        is_chimera_small = False
        for filename in file_names:
            self.assertTrue('sampledata' not in filename and 'testDataFiles' not in filename,
                            msg="Filename should include path."
                                " Was '{0}'.".format(filename))
            if 'chimera_small.log' == filename:
                is_chimera_small = True

        self.assertTrue(is_chimera_small, msg="File {0} was not found, but was expected.".format('chimera_small.log'))

    def test_sorted(self):
        """
        Make sure the list returned is sorted.
        """
        file_names_should_be = []

        for (dir_path, dir_names, file_names) in os.walk(TEST_FILES_PATH):
            file_names_should_be.extend(file_names)

        file_names = tf.get_all_file_names(with_path=False)

        self.assertEqual(file_names, sorted(file_names_should_be),
                         msg="List not sorted.\nShould"
                             " be {0}.\nWas {1}.".format(file_names_should_be, file_names))




from pyporegui.version import version as __version__


def start():
    """
    Starts PyporeGui.

    >>> import pyporegui
    >>> pyporegui.start()
    """
    from pyporegui.gui import start as _start
    _start()



"""
Functions that return file names of data files used for testing.
"""
import os


dir_name = os.path.dirname(os.path.abspath(__file__))
TEST_DATA_FOLDER = 'testDataFiles'
TEST_DATA_FOLDER_PATH = os.path.join(dir_name, TEST_DATA_FOLDER)


class TestFileDoesntExistError(Exception):
    pass


def get_all_file_names(with_path=True):
    """
    :param bool with_path: Whether the returned file names should include their absolute paths. Default is True.
    :returns: A list of all of the file names in the testDataFiles directory.
    """
    ret_file_names = []

    for (dir_path, dir_names, file_names) in os.walk(TEST_DATA_FOLDER_PATH):
        if with_path:
            for i, filename in enumerate(file_names):
                file_names[i] = os.path.join(TEST_DATA_FOLDER_PATH, filename)
        ret_file_names.extend(file_names)

    return sorted(ret_file_names)


def get_abs_path(filename):
    """
    :returns: The absolute path of the test data file name.
    :raises: :py:exc:`TestFileDoesntExistError` if the filename is not in the test folder.
    """
    abs_path = os.path.abspath(os.path.join(TEST_DATA_FOLDER_PATH, filename))
    if not os.path.exists(abs_path):
        raise TestFileDoesntExistError(
            "Test file '{0}' does not exist under directory '{1}'.".format(filename, TEST_DATA_FOLDER_PATH))
    return abs_path

# TODO add function for searching file names by keyword.



#!/usr/bin/env python
import argparse
import fnmatch
from multiprocessing import Pool
import os
from pypore.filetypes.event_database import open_file

def _filter_wrap(tup):
    # returns the output filename
    filename = tup[0]
    print "Starting file:", filename
    ed = open_file(filename, mode='r')
    output_filename = filename[:-3] + '.csv'
    ed.to_csv(output_filename)
    print "Done converting input {0} into output {1}.".format(tup[0], output_filename)
    return output_filename


def main():
    """

    * -h, --help            show this help message and exit
    * -d DIRECTORY, --directory DIRECTORY
        directory of files to filter. Default is the current directory.
    * -r, --recursive       search for files recursively.

    """
    filter_freq = 1.e4
    re_sample_freq = 1.e5
    glob_search = '*Events*.h5'

    # parse the command line arguments
    parser = argparse.ArgumentParser(description="Filters files in a directory based on a file extension.")
    parser.add_argument('-d', '--directory', type=str, nargs=1,
                        help="directory of files to filter. Default is the current directory.")
    parser.add_argument('-r', '--recursive', action='store_true',
                        help="search for files recursively.")
    args = parser.parse_args()

    directory = '.'
    # Use the command line arguments to set our variables, if necessary.
    if args.directory is not None:
        directory = args.directory[0]

    # find all of the files in the current directory with .h5 extension.
    files = []
    for root, dirname, filenames in os.walk(directory):
        for filename in fnmatch.filter(filenames, glob_search):
            files.append(os.path.join(root, filename))
        # Only do top level directory, unless recursive is specified.
        if not args.recursive:
            break

    print "Filter frequency: {0} Hz".format(filter_freq)
    print "Output sample frequency: {0} Hz".format(re_sample_freq)
    print "Glob search: {0}".format(glob_search)
    print "Recursive: {0}".format(args.recursive)
    print "Filtering these files:", files
    print "\n----------------------------\n"

    p = Pool()

    # add the file names and filter frequency and output sample rate to a tuple to pass in multiprocessing
    pool_args = []
    for filename in files:
        tup = (filename,)
        pool_args.append(tup)

    # filter each file
    output_file_names = p.map(_filter_wrap, pool_args)

    print "\n----------------------------\n"
    print "Output files:", output_file_names

if __name__ == '__main__':
    main()



"""
Module for creating sample data.

- Create a current trace with specified events using \
    :func:`create_specified_data <pypore.sampledata.creator.create_specified_data>`.
- Create a current trace with random events using \
    :func:`create_random_data <pypore.sampledata.creator.create_random_data>`.
"""

import os
import numpy as np
from scipy.stats import geom

from pypore.filetypes.data_file import open_file


def create_random_data(filename, seconds, sample_rate, baseline=0.0, noise=None, event_rate=0, event_durations=None,
                       event_depths=None, overwrite=False):
    """
    Creates random sample data. Leaves the first 200 data points free of events.

    :param str filename: Filename for the data. If the file already exists and overwrite=False, an IOError is raised.
    :param float seconds: Number of seconds of data.
    :param float sample_rate: The sampling rate of the data, in Hz.
    :param float baseline: The baseline of the data, in uA.
    :param scipy.stats.distributions.rv_frozen noise: A frozen :mod:`scipy.stats` probability distribution\
            for the noise. An example normal distribution with mean 2 uA and std dev 3 uA is
            ::

                from scipy.stats import norm
                noise = norm(loc=2, scale=3)

            Default is no noise.
    :param float event_rate: Rate of events in Hz.
    :param scipy.stats.distributions.rv_frozen event_durations: A frozen :mod:`scipy.stats` probability distribution\
            for the event duration (in seconds).
    :param scipy.stats.distributions.rv_frozen event_depths: A frozen :mod:`scipy.stats` probability distribution\
            for the event depth, in uA.
    :param bool overwrite: Whether overwriting an existing file at filename is allowed. If false, and filename exists.\
            an IOError will be raised.
    :raises: :py:exc:`IOError` - If the filename already exists and overwrite=False.

    >>> from pypore.sampledata.creator import create_random_data
    >>> from scipy import stats
    >>> seconds = 1. # 1 second of data.
    >>> sample_rate = 1.e6 # 1 MHz sample rate.
    >>> event_rate = 100. # 100 events/sec, on average.
    >>> baseline = 10. # 10 uA baseline.
    >>> noise = stats.norm(scale=.5) # Normal distributed noise with mean of 0 std dev of 0.5 uA.
    >>> event_depths = stats.norm(loc=2., scale=1.) # Normal distributed events with mean of 2 and std dev of 1 uA.
    >>> event_durations = stats.norm(loc=100.e-6, scale=10.e-6) # Normal distributed event durations with mean of 100 us and std dev 10 us.
    >>> n_events = create_random_data('random_trace.h5', seconds, sample_rate, baseline, noise, event_rate, event_durations, event_depths)
    """
    if not overwrite and os.path.exists(filename):
        raise IOError(
            "File already exists. Use a different filename, or call with overwrite=True to over-write existing file.")

    n_points = int(seconds * sample_rate)
    f = open_file(filename, mode='w', n_points=n_points, sample_rate=sample_rate)

    data = np.zeros(n_points) + baseline

    if noise is not None:
        data += noise.rvs(size=n_points)

    event_count = 0
    if event_rate > 0:
        i = 200
        mean_length = event_durations.mean() * sample_rate
        expected_events = seconds * event_rate
        # Available space that is not events or the beginning of the data.
        free_space = n_points - expected_events * mean_length - i
        event_probability = expected_events/free_space
        # Use geometric distribution to find the next starting spot of an event.
        rv = geom(event_probability)
        while i < n_points:
            # get next event start distance
            i += rv.rvs()
            event_count += 1
            event_length = event_durations.rvs() * sample_rate
            event_depth_i = event_depths.rvs()
            if i + event_length > n_points:
                event_length = n_points - i
            data[i:i+event_length] += event_depth_i
            i += event_length

    f.root.data[:] = data[:]

    f.close()

    return event_count


def create_specified_data(filename, n, sample_rate, baseline=0.0, noise_scale=0.0, events=np.zeros(0), overwrite=False):
    """
    Creates a :class:`DataFile <pypore.filetypes.data_file.DataFile>` with sample event data.

    :param filename: The name for the new DataFile.
    :param int n: The number of samples in the data set.
    :param float sample_rate: The sample rate of the data set.
    :param float baseline: The baseline of the data. Default is 0.0.
    :param float noise_scale: Passed to :func:`numpy.random.normal`, it is the standard deviation of the Gaussian\
            noise added to the signal. Default is 0.0.
    :param events: A list of events to add to the baseline. Events are specified as\
            [starting_point, length, current_change]. So for example
            ::

                events = [[100,200,-.1], [560, 100, .6]]

            specifies 2 events:

        #. Starts at data position 100, with length 200, and current change of -.1.
        #. Starts at data position 560, with length of 100, and current change of +.6.

        Default is no events added.

    :param bool overwrite: Whether to overwrite an existing file at **filename**, if it exists. If **False**, and\
            filename already exists, an IOError will be raised. Default **False**.
    :raises: :exc:`IOError` - If the filename already exists and overwrite=False.

    >>> from pypore.sampledata.creator import create_specified_data as csd
    >>> filename = "test.h5"
    >>> n = 10000
    >>> events = [[100, 200, -.5], [3000, 1000, -.8]] # Creates an event starting at position 100, with length 200 and\
                                                      #  baseline change of -0.5.
    >>> create_specified_data(filename, n, 1.e4, baseline=5.0, noise_scale=1.0, events=events)
    """
    if not overwrite and os.path.exists(filename):
        raise IOError(
            "File already exists. Use a different filename, or call with overwrite=True to over-write existing file.")

    f = open_file(filename, mode='w', n_points=n, sample_rate=sample_rate)

    data = np.zeros(n) + baseline

    if noise_scale > 0:
        data += np.random.normal(scale=noise_scale, size=n)

    for event in events:
        data[event[0]:event[0] + event[1]] += event[2]

    f.root.data[:] = data[:]

    f.close()



from multiprocessing import Pool
import unittest
import os

import scipy.stats as stats
import numpy as np
from pypore.event_finder import find_events

from pypore.i_o import get_reader_from_filename
from pypore.sampledata.creator import create_specified_data
from pypore.sampledata.creator import create_random_data
from pypore.filetypes.event_database import EventDatabase
from pypore.tests.util import _test_file_manager

DIRECTORY = os.path.dirname(os.path.abspath(__file__))


def _test_params_equality(self, filename, data_should_be, sample_rate):
    """
    Tests that the data and sample_rate in filename equal data_should_be and sample_rate, respectively.
    """
    reader = get_reader_from_filename(filename)
    data_all = reader.get_all_data()
    out_data = data_all[0]
    out_sample_rate = reader.get_sample_rate()
    reader.close()

    self.assertEqual(sample_rate, out_sample_rate,
                     "Sample rates not equal. Wanted {0}, got {1}.".format(sample_rate, out_sample_rate))

    np.testing.assert_array_equal(out_data, data_should_be)


def _test_number_of_events_help(name):
    # Need to seed, otherwise each process will generate the same random data!
    np.random.seed()
    # Check that setting the event rate gives correct number of events.
    seconds = 5.
    sample_rate = 1.e6
    baseline = 1.
    event_rate = 50.
    event_duration = stats.norm(loc=100.e-6, scale=5.e-6)
    event_depth = stats.norm(loc=-1., scale=.05)
    noise = stats.norm(scale=0.02)
    # create a list of file_names so we can average the number of events.
    if os.path.exists(name):
        os.remove(name)
    n_e_r = create_random_data(filename=name, seconds=seconds, sample_rate=sample_rate,
                               baseline=baseline, noise=noise,
                               event_rate=event_rate, event_durations=event_duration,
                               event_depths=event_depth)

    save_file_name = name[:-len('.h5')] + '_Events.h5'
    if os.path.exists(save_file_name):
        os.remove(save_file_name)
    event_database = find_events([name], save_file_names=[save_file_name])[0]

    ed = EventDatabase(event_database)

    n_events_found = ed.get_event_count()

    os.remove(name)
    ed.close()
    os.remove(event_database)

    return n_e_r, n_events_found, name


class TestCreateRandomData(unittest.TestCase):
    @_test_file_manager('.')
    def test_output_filename_correct(self, filename):
        """
        Tests that the output filename is correct.
        """
        create_random_data(filename, seconds=1., sample_rate=10.)

        self.assertTrue(os.path.exists(filename))

    @_test_file_manager(DIRECTORY)
    def test_opening_existing_filename_raises(self, filename):
        """
        Tests that trying to open an existing filename without setting overwrite=True will raise an IOError.
        """
        # create a blank file
        f = open(filename, mode='w')
        f.close()

        self.assertRaises(IOError, create_random_data, filename, seconds=1., sample_rate=10.)

    @_test_file_manager(DIRECTORY)
    def test_opening_existing_with_overwrite(self, filename):
        """
        Tests that we can overwrite an existing file with overwrite=True
        """
        # create a blank file
        f = open(filename, mode='w')
        f.close()

        seconds = 1.
        sample_rate = 1.
        baseline = 1.

        create_random_data(filename, seconds=seconds, sample_rate=sample_rate, baseline=baseline, overwrite=True)

        self.assertTrue(os.path.exists(filename))

    @_test_file_manager(DIRECTORY)
    def test_baseline_no_noise_no_events(self, filename):
        """
        Tests that with only the baseline, we create the right data.
        """
        seconds = 1.
        sample_rate = 1.e6
        baseline = 1.

        event_rate = 0

        event_duration = stats.norm(loc=100.e-6, scale=10.e-6)
        event_depth = stats.norm(loc=.5, scale=.05)

        data_should_be = np.zeros(int(seconds * sample_rate)) + baseline

        create_random_data(filename=filename, seconds=seconds, sample_rate=sample_rate, baseline=baseline,
                           event_rate=event_rate, event_durations=event_duration, event_depths=event_depth)

        _test_params_equality(self, filename, data_should_be, sample_rate)

    @_test_file_manager(DIRECTORY)
    def test_number_of_events(self, filename):
        """
        Tests that the number of events is roughly correct. The number of events should be random, which is why
        only roughly.
        """
        # NOTE: The below parameters are copied above in _test_number_of_events_help, so
        # changing these below will not do anything! See above!
        seconds = 5.
        sample_rate = 1.e6
        baseline = 1.
        event_rate = 50.
        event_duration = stats.norm(loc=100.e-6, scale=5.e-6)
        event_depth = stats.norm(loc=-1., scale=.05)
        noise = stats.norm(scale=0.02)
        n_events_should_be = event_rate * seconds

        # Check that setting the event rate gives correct number of events.

        n_events_returned_arr = []
        n_events_found_arr = []

        trials = 15
        p = Pool()

        args = []
        # create a list of file_names so we can average the number of events.
        for i in xrange(10):
            name = filename[:-len('.h5')] + '_' + str(i) + '.h5'
            args.append(name)

        x = p.map(_test_number_of_events_help, args)

        print "result:", x

        for thing in x:
            print "ner:", thing[0], "nef:", thing[1], "name:", thing[2]
            n_events_returned_arr.append(thing[0])
            n_events_found_arr.append(thing[1])

        mean_event_returned = np.mean(n_events_returned_arr)
        difference = abs(mean_event_returned - n_events_should_be)
        self.assertLessEqual(difference, 10,
                             "Unexpected mean number of events returned from function. "
                             "Should be {0}, was {1}.".format(n_events_should_be,
                                                              mean_event_returned))

        mean_events_found = np.mean(n_events_found_arr)
        difference = abs(mean_events_found - mean_event_returned)
        self.assertLessEqual(difference, 5,
                             "Unexpected mean number of events found. "
                             "Should be {0}, was {1}.".format(mean_event_returned,
                                                              mean_events_found))

    @_test_file_manager(DIRECTORY)
    def test_noise(self, filename):
        """
        Tests that the noise is added correctly(its mean and std_dev are correct).
        """
        seconds = 1.
        sample_rate = 1.e6
        baseline = 1.
        noise_loc = 1.
        noise_scale = 0.6
        noise = stats.norm(loc=noise_loc, scale=noise_scale)

        create_random_data(filename, seconds=seconds, sample_rate=sample_rate, baseline=baseline,
                           noise=noise)

        reader = get_reader_from_filename(filename)
        data_all = reader.get_all_data()
        data = data_all[0]

        mean = np.mean(data)
        mean_should_be = baseline + noise_loc
        self.assertAlmostEqual(mean, mean_should_be, 1,
                               "Unexpected mean. Wanted {0}, got {1}.".format(mean_should_be, mean))

        std_dev = np.std(data)
        self.assertAlmostEqual(noise_scale, std_dev, 1, "Unexpected standard deviation.  "
                                                        "Wanted {0}, got {1}".format(noise_scale, std_dev))
        reader.close()

    @_test_file_manager(DIRECTORY)
    def test_event_params(self, filename):
        """
        Tests that setting the event depth and duration give correct values.
        """
        seconds = 5.
        sample_rate = 1.e6
        baseline = 10.
        event_rate = 100.
        event_duration_loc = 1.e-4
        event_duration_scale = 5.e-6
        event_duration = stats.norm(loc=event_duration_loc, scale=event_duration_scale)
        event_depth_loc = -1.
        event_depth_scale = .05
        event_depth = stats.norm(loc=-1., scale=.05)
        noise = stats.norm(scale=0.01)

        n_events_returned = create_random_data(filename, seconds=seconds, sample_rate=sample_rate,
                                               baseline=baseline, event_rate=event_rate,
                                               event_durations=event_duration, event_depths=event_depth,
                                               noise=noise)

        save_file_name = filename[:-len('.h5')] + 'Events.h5'
        if os.path.exists(save_file_name):
            os.remove(save_file_name)
        event_database = find_events([filename], save_file_names=[save_file_name])[0]

        ed = EventDatabase(event_database)

        count = ed.get_event_count()
        count_should_be = event_rate * seconds
        diff = abs(count - count_should_be)
        self.assertLessEqual(diff, 100, "Unexpected number of events. "
                                        "Expected {0}, was {1}.".format(count_should_be, count))

        table_sample_rate = ed.get_sample_rate()
        durations = [x['event_length'] / table_sample_rate for x in ed.get_event_table().iterrows()]
        depths = [x['current_blockage'] for x in ed.get_event_table().iterrows()]

        mean_duration = np.mean(durations)
        self.assertAlmostEqual(event_duration_loc, mean_duration, 5, "Unexpected mean event duration.  "
                                                                     "Wanted {0}, got {1}.".format(event_duration_loc,
                                                                                                   mean_duration))

        mean_depth = np.mean(depths)
        self.assertAlmostEqual(event_depth_loc, mean_depth, 1, "Unexpected mean event depth. "
                                                               "Wanted {0}, got {1}.".format(event_depth_loc,
                                                                                             mean_depth))

        ed.close()
        os.remove(event_database)


class TestCreateSpecifiedData(unittest.TestCase):
    @_test_file_manager('.')
    def test_output_filename_correct(self, filename):
        create_specified_data(filename, n=100, sample_rate=10)

        self.assertTrue(os.path.exists(filename))

    @_test_file_manager(DIRECTORY)
    def test_opening_existing_filename_raises(self, filename):
        """
        Tests that trying to open an existing filename raises an exception.
        """
        # create a blank file
        f = open(filename, mode='w')
        f.close()

        # try to create a data set on the same file
        self.assertRaises(IOError, create_specified_data, filename, 100, 100.)

    @_test_file_manager(DIRECTORY)
    def test_opening_existing_filename_overwrite(self, filename):
        """
        Tests that we can overwrite an existing filename if we use the overwrite parameter.
        """
        # create a blank file
        f = open(filename, mode='w')
        f.close()

        sample_rate = 1.e6
        baseline = 0.2
        n = 5000

        data_should_be = np.zeros(n) + baseline

        create_specified_data(filename=filename, n=n, sample_rate=sample_rate, baseline=baseline, noise_scale=0.0,
                              events=[], overwrite=True)

        _test_params_equality(self, filename=filename, data_should_be=data_should_be, sample_rate=sample_rate)

    @_test_file_manager(DIRECTORY)
    def test_simple_baseline(self, filename):
        """
        Tests that a no noise, no event returns just a baseline.
        """
        sample_rate = 1.e6
        baseline = 0.2
        n = 5000

        data_should_be = np.zeros(n) + baseline

        create_specified_data(filename=filename, n=n, sample_rate=sample_rate, baseline=baseline, noise_scale=0.0,
                              events=[])

        _test_params_equality(self, filename=filename, data_should_be=data_should_be, sample_rate=sample_rate)

    @_test_file_manager(DIRECTORY)
    def test_no_noise_specified_events(self, filename):
        """
        Tests that with no noise and specified events we get the right matrix
        """
        sample_rate = 1.e6
        baseline = 1.0
        n = 5000

        events = [[100, 200., -.5], [1000, 500, -1.0], [3000, 1000, .2]]

        data_should_be = np.zeros(n) + baseline

        # add in the events to the data_should_be
        for event in events:
            data_should_be[event[0]:event[0] + event[1]] += event[2]

        create_specified_data(filename=filename, n=n, sample_rate=sample_rate, baseline=baseline, noise_scale=0.0,
                              events=events)

        _test_params_equality(self, filename=filename, data_should_be=data_should_be, sample_rate=sample_rate)

    @_test_file_manager(DIRECTORY)
    def test_noise_no_events(self, filename):
        """
        Tests that noise is added correctly.
        """
        sample_rate = 1.e5
        baseline = 1.0
        n = 100000
        noise_scale = 1.

        create_specified_data(filename=filename, n=n, sample_rate=sample_rate, baseline=baseline,
                              noise_scale=noise_scale, events=[])

        reader = get_reader_from_filename(filename)
        data_all = reader.get_all_data()
        data = data_all[0]
        reader.close()

        self.assertEqual(n, data.size, "Unexpected array size. Wanted {0}, got {1}.".format(n, data.size))
        decimal = 1
        mean = np.mean(data)
        self.assertAlmostEqual(baseline, mean, decimal,
                               "Unexpected baseline. Wanted {0}, got {1}. Tested to {2} decimals.".format(baseline,
                                                                                                          mean,
                                                                                                          decimal))
        decimal = 1
        std_dev = np.std(data)
        self.assertAlmostEqual(noise_scale, std_dev, decimal,
                               msg="Unexpected stddev. Wanted{0}, got {1}. Tested to {2} decimals.".format(noise_scale,
                                                                                                           std_dev,
                                                                                                           decimal))



import unittest

from pypore.i_o.heka_reader import HekaReader
from pypore.i_o.tests.reader_tests import ReaderTests
import pypore.sampledata.testing_files as tf


class TestHekaReader(unittest.TestCase, ReaderTests):
    reader_class = HekaReader

    default_test_data_files = [tf.get_abs_path('heka_1.5s_mean5.32p_std2.76p.hkd')]

    def help_scaling(self):
        filename = tf.get_abs_path('heka_1.5s_mean5.32p_std2.76p.hkd')
        mean = 5.32e-12
        std_dev = 2.76e-12
        return [filename], [mean], [std_dev]

    @unittest.skip("Test file is too short for decimated and un-decimated means to be equal enough.")
    def test_scaling_decimated(self):
        super(TestHekaReader, self).test_scaling_decimated()




import unittest
import numpy as np
import os
import csv

from pypore.i_o.cnp2_reader import CNP2Reader
from pypore.i_o.tests.reader_tests import ReaderTests
import pypore.sampledata.testing_files as tf


class TestChimeraReader(unittest.TestCase, ReaderTests):
    reader_class = CNP2Reader

    default_test_data_files = [tf.get_abs_path('cnp_test.hex')]

    def test_constructor_no_mat_spec(self):
        """
        Tests that an IOError is raised when no .cfg config file is next to the .hex file.
        """
        test_no_cfg_file = tf.get_abs_path('cnp_empty.hex')
        for filename in test_no_cfg_file:
            self.assertRaises(IOError, CNP2Reader, filename)

    def test_get_all_data(self):
        # Make sure path to file is correct.
        filename = tf.get_abs_path('cnp_test.hex')
        cnp_reader = CNP2Reader(filename)
        data = cnp_reader.get_all_data(False)[0]

        csv_filename = filename[:-4] + '.csv'
        csv_data = self._get_test_csv_data(csv_filename)

        ratio = np.abs((csv_data-data) / csv_data)

        np.testing.assert_array_almost_equal(ratio, np.zeros_like(data))

        cnp_reader.close()

    def _get_test_csv_data(self, csv_filename):
        data = []
        with open(csv_filename, 'rb') as csvfile:
            csvreader = csv.reader(csvfile, delimiter=',')
            # skip first row
            csvreader.next()
            for row in csvreader:
                data.append(float(row[1]))
        return np.array(data) * 1.e9 # scale the data to nA

    def help_scaling(self):
        filename = tf.get_abs_path('cnp_test.hex')
        mean_should_be = 10.363  # Value gotten from the exported CSV file
        std_should_be = 4.6866  # Value gotten from the exported CSV file
        return [filename], [mean_should_be], [std_should_be]

    def help_scaling_decimated(self):
        filename = tf.get_abs_path('cnp_test.hex')
        return [filename]


if __name__ == "__main__":
    unittest.main()



from pyqtgraph.graphicsItems.ScatterPlotItem import ScatterPlotItem
from pyporegui.graphicsItems.spot_item import SpotItem


class ScatterPlotItem(ScatterPlotItem):

    def __init__(self, *args, **kargs):
        super(ScatterPlotItem, self).__init__(*args, **kargs)
        self.files = kargs['files']
        self.counts = kargs['counts']  # number of events in each file

    def points(self):
        for i, rec in enumerate(self.data):
            if rec['item'] is None:
                rec['item'] = SpotItem(rec, self, i)
        return self.data['item']

    def get_file_name_from_position(self, position):
        """
        Returns filename, eventNumber for the
        SpotItem position.
        """
        for i, j in enumerate(self.counts):
            if position < j:
                return self.files[i], position
            position -= j
        return None


from PySide import QtGui
import numpy as np
from pyqtgraph import GraphicsLayoutWidget, mkPen
from pypore.filetypes import event_database as ed
from pyporegui.graphicsItems.histogram_item import HistogramItem
from pyporegui.graphicsItems.scatter_plot_item import ScatterPlotItem

__all__ = ['EventAnalysisPlotWidget']


class EventAnalysisPlotWidget(GraphicsLayoutWidget):
    def __init__(self):
        super(EventAnalysisPlotWidget, self).__init__()

        self.plot_event_depth = HistogramItem(title='Event Depth', rotate=True)
        self.addItem(self.plot_event_depth)
        self.plot_event_depth.setMouseEnabled(x=False, y=True)
        self.plot_event_dur_event_depth = self.addPlot(name='Depth vs. Duration', title='Depth vs. Duration')
        self.plot_event_depth.setYLink('Depth vs. Duration')

        self.nextRow()

        self.plot_scatter_select = self.addPlot(title='Single Event')

        self.plot_event_dur = HistogramItem(title='Event Duration')
        self.addItem(self.plot_event_dur)
        self.plot_event_dur.setXLink('Depth vs. Duration')
        self.plot_event_dur.setMouseEnabled(x=True, y=False)

        self.last_scatter_clicked = []

        self.n_bins = 0
        self.bins = np.zeros(0)

    def add_selections(self, file_names, params):
        """
        Plots event statistics.
        """
        files = []
        counts = []
        event_count = 0
        for filename in file_names:
            h5file = ed.open_file(filename, mode='r')
            files.append(h5file)
            count = h5file.get_event_count()
            event_count += count
            counts.append(count)

        current_blockade = np.empty(event_count)
        dwell_times = np.empty(event_count)
        count = 0
        for j, filex in enumerate(files):
            event_table = filex.get_event_table()
            sample_rate = filex.get_sample_rate()
            for i, row in enumerate(event_table):
                current_blockade[count + i] = row['current_blockage']
                dwell_times[count + i] = row['event_length'] / sample_rate
            count += counts[j]

        color = params['color']
        new_color = QtGui.QColor(color.red(), color.green(), color.blue(), 128)

        self.plot_event_dur.add_histogram(dwell_times, color=new_color)

        self.plot_event_depth.add_histogram(current_blockade, color=new_color)

        scatter_item = ScatterPlotItem(size=10, pen=mkPen(None), brush=new_color, files=file_names, counts=counts)
        scatter_item.setData(dwell_times, current_blockade)
        self.plot_event_dur_event_depth.addItem(scatter_item)
        scatter_item.sigClicked.connect(self.on_scatter_points_clicked)

        for filex in files:
            filex.close()

        return

    def remove_filter(self, index):
        self.plot_event_dur.remove_item_at(index)
        self.plot_event_depth.remove_item_at(index)
        self.plot_event_dur_event_depth.removeItem(self.plot_event_dur_event_depth.listDataItems()[index])

    def on_scatter_points_clicked(self, plot, points):
        """
        Callback for when a scatter plot points are clicked.
        Highlights the points and un-highlights previously selected points.

        plot should be a MyScatterPlotItem
        points should be a MySpotItem
        """
        for p in self.last_scatter_clicked:
            p.resetPen()
            # remove point we've already selected so we
            # can select points behind it.
            if p in points and len(points) > 1:
                points.remove(p)
            #         print 'Points clicked:', points, plot
        for point in points:
            point.setPen('w', width=2)
            self.last_scatter_clicked = [point]
            break  # only take first point

        # Plot the new point clicked on the single event display
        filename, position = plot.get_file_name_from_position(self.last_scatter_clicked[0].event_position)

        h5file = ed.open_file(filename, mode='r')

        row = h5file.get_event_row(position)
        array_row = row['array_row']
        sample_rate = h5file.get_sample_rate()
        event_length = row['event_length']
        raw_points_per_side = row['raw_points_per_side']

        raw_data = h5file.get_raw_data_at(array_row)

        n = len(raw_data)

        times = np.linspace(0.0, 1.0 * n / sample_rate, n)

        self.plot_scatter_select.clear()
        self.plot_scatter_select.plot(times, raw_data)
        # plot the event points in yellow
        self.plot_scatter_select.plot(times[raw_points_per_side:raw_points_per_side + event_length],
                                      raw_data[raw_points_per_side:raw_points_per_side + event_length], pen='y')

        # Plot the cusum levels
        n_levels = row['n_levels']
        baseline = row['baseline']
        # left, start-1, start,
        levels = h5file.get_levels_at(array_row)
        indices = h5file.get_level_lengths_at(array_row)

        level_times = np.zeros(2 * n_levels + 4)
        level_values = np.zeros(2 * n_levels + 4)

        level_times[1] = 1.0 * (raw_points_per_side - 1) / sample_rate
        level_values[0] = level_values[1] = baseline
        i = 0
        length = 0
        for i in xrange(n_levels):
            level_times[2 * i + 2] = times[raw_points_per_side] + 1.0 * length / sample_rate
            level_values[2 * i + 2] = levels[i]
            level_times[2 * i + 3] = times[raw_points_per_side] + 1.0 * (length + indices[i]) / sample_rate
            level_values[2 * i + 3] = levels[i]
            length += indices[i]
        i += 1
        level_times[2 * i + 2] = times[raw_points_per_side + event_length]
        level_times[2 * i + 3] = times[n - 1]
        level_values[2 * i + 2] = level_values[2 * i + 3] = baseline

        self.plot_scatter_select.plot(level_times, level_values, pen='g')

        h5file.close()


from PySide import QtGui
from pyqtgraph.graphicsItems.PlotItem import PlotItem
from pyqtgraph.widgets.LayoutWidget import LayoutWidget
import pyqtgraph as pg
from pyporegui.widgets.plot_tool_bar import PlotToolBar

from base_tabs import BaseQSplitterDataFile

__all__ = ['FileViewerTab']


class FileViewerTab(BaseQSplitterDataFile):
    """

    """

    def __init__(self, parent=None):
        """

        """
        super(FileViewerTab, self).__init__(parent)

    def _on_file_item_selection_changed(self):
        # TODO implement me
        # or choose to do nothing
        pass

    def _on_file_item_doubleclick(self, item):
        # TODO implement me
        pass

    def _create_right_widget(self, parent):
        plot_wig = pg.GraphicsLayoutWidget()

        self.plot_widget = PlotItem(name='FileViewer', title='Current')

        plot_wig.addItem(self.plot_widget)

        self.plot_tool_bar = PlotToolBar(self)
        if parent is not None:
            parent.addToolBar(self.plot_tool_bar)

        layout_wig = LayoutWidget()
        layout_wig.addWidget(self.plot_tool_bar, row=1, col=0, colspan=3)
        layout_wig.addWidget(plot_wig, row=2, col=0, colspan=3)

        return layout_wig

    def _create_left_widget(self, parent=None):
        scroll_area = QtGui.QScrollArea()
        scroll_area.setWidgetResizable(True)

        file_options = super(FileViewerTab, self)._create_left_widget(parent)

        filter_frame = QtGui.QFrame()

        v_box = QtGui.QVBoxLayout()
        v_box.addLayout(file_options)
        v_box.addWidget(filter_frame)

        v_box_widget = QtGui.QWidget()
        v_box_widget.setLayout(v_box)

        scroll_area.setWidget(v_box_widget)

        return scroll_area



from PySide import QtGui
import numpy as np
from pyqtgraph import PlotItem
from pyqtgraph.graphicsItems.PlotCurveItem import PlotCurveItem


class HistogramItem(PlotItem):
    def __init__(self, *args, **kwargs):
        self.rotate = False
        if 'rotate' in kwargs:
            self.rotate = kwargs['rotate']
            del kwargs['rotate']  # if you don't delete, MyHistogramItem will start with
            # a PlotDataItem for some reason

        super(HistogramItem, self).__init__(*args, **kwargs)

        self.data_array = []
        self.bins = np.zeros(1)
        self.n_bins = 0
        self.minimum = 0.
        self.maximum = 0.

        self.minimum_array = []
        self.maximum_array = []
        self.n_bins_array = []

    def add_histogram(self, data, n_bins=None, color=None):
        """
        Adds a histogram to the plot.

        :param numpyArray data: Data for histogram.
        :param IntType n_bins: Number of bins for the histogram. Default value is sqrt(data.size).
        :param QtGui.QColor color: (Optional) color of the new histogram.
        """
        if n_bins is None:
            n_bins = data.size ** 0.5

        minimum = data.min()
        maximum = data.max()

        self.minimum_array.append(minimum)
        self.maximum_array.append(maximum)
        self.n_bins_array.append(n_bins)

        # if this is the first histogram plotted,
        # initialize_c settings
        if len(self.data_array) < 1:
            self.minimum = minimum
            self.maximum = maximum
            self.n_bins = n_bins
            self.bins = np.linspace(self.minimum, self.maximum, int(self.n_bins + 1))

        # re-plot the other histograms with this new
        # binning if needed
        re_hist = False
        if minimum < self.minimum:
            self.minimum = minimum
            re_hist = True
        if maximum > self.maximum:
            self.maximum = maximum
            re_hist = True
        if n_bins > self.n_bins:
            self.n_bins = n_bins
            re_hist = True

        if re_hist:
            self._re_histogram()

        self._plot_histogram(data, color)

    def _plot_histogram(self, data, color=None):
        if color is None:
            color = QtGui.QColor(0, 0, 255, 128)

        # Copy self.bins, otherwise it is returned as x, which we can accidentally modify
        # by x *= -1, leaving self.bins modified.
        bins = self.bins.copy()
        y, x = np.histogram(data, bins=bins)

        if self.rotate:
            x *= -1.
        curve = PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=color)
        if self.rotate:
            curve.rotate(-90)
        self.addItem(curve)

        self.data_array.append(data)

    def _re_histogram(self):
        self.bins = np.linspace(self.minimum, self.maximum, self.n_bins + 1)
        items = self.listDataItems()
        for i, item in enumerate(items):
            # Copy self.bins, otherwise it is returned as x, which we can accidentally modify
            # by x *= -1, leaving self.bins modified.
            bins = self.bins.copy()
            y, x = np.histogram(self.data_array[i], bins=bins)
            if self.rotate:
                x *= -1.
            item.setData(x, y)

    def remove_item_at(self, index):
        """
        Removes the histogram item at index.

        :param IntType index: Index of the histogram to remove.
        """
        if len(self.data_array) < 1:
            return

        self.removeItem(self.listDataItems()[index])
        del self.data_array[index]
        del self.minimum_array[index]
        del self.maximum_array[index]
        del self.n_bins_array[index]

        # return if no more histograms to display
        if len(self.data_array) < 1:
            return

        re_hist = False
        # do we need to re-plot?
        maxi = max(self.maximum_array)
        if maxi < self.maximum:
            re_hist = True
            self.maximum = maxi
        mini = min(self.minimum_array)
        if mini > self.minimum:
            re_hist = True
            self.minimum = mini
        nb = max(self.n_bins_array)
        if nb < self.n_bins:
            re_hist = True
            self.n_bins = nb

        if re_hist:
            self._re_histogram()


import os
import unittest
import numpy as np

from pypore.tests.util import _test_file_manager
import pypore.sampledata.testing_files as tf
from pypore.filter_files import filter_file
from pypore.i_o import get_reader_from_filename

DIRECTORY = os.path.dirname(os.path.abspath(__file__))


class TestFilterFile(unittest.TestCase):
    def setUp(self):
        self.directory = os.path.dirname(os.path.abspath(__file__))

    def test_default_output_name(self):
        """
        Tests that the default output filename of :func:`filter_file <pypore.file_converter.filter_file>` is correct.
        """
        data_filename = tf.get_abs_path('chimera_1event.log')

        output_filename_should_be = os.path.join(tf.TEST_DATA_FOLDER_PATH, 'chimera_1event_filtered.h5')

        if os.path.exists(output_filename_should_be):
            os.remove(output_filename_should_be)

        out_filename = filter_file(data_filename, 10.e4, 100.e4)

        self.assertEqual(out_filename, output_filename_should_be,
                         msg="Default filename incorrect. Was {0}, should be {1}.".format(out_filename,
                                                                                          output_filename_should_be))

        # Make sure the file actually exists. Try opening it.
        f = open(out_filename)
        f.close()

        os.remove(out_filename)

    @_test_file_manager(DIRECTORY)
    def test_set_output_name(self, filename):
        """
        Tests that setting the output filename of :func:`filter_file <pypore.file_converter.filter_file>` is correct.
        """
        data_filename = tf.get_abs_path('chimera_1event.log')

        out_filename = filter_file(data_filename, 10.e4, 100.e4, output_filename=filename)

        self.assertEqual(filename, out_filename,
                         "Output filename not set correctly. Was {0}, should be {1}".format(filename,
                                                                                            out_filename))

        # Make sure the file actually exists. Try opening it.
        f = open(out_filename)
        f.close()

    def _test_out_sample_rate_data_len_equality(self, orig_data, orig_sample_rate, out_filename, sample_rate):
        out_reader = get_reader_from_filename(out_filename)
        out_sample_rate = out_reader.get_sample_rate()
        out_data = out_reader.get_all_data()
        out_reader.close()
        self.assertAlmostEqual(orig_sample_rate, out_sample_rate, 2,
                               msg="Sampling rate changed during filter_file, "
                                   "when it should not. Was {0}, wanted {1}, got {2}".format(
                                   orig_sample_rate, sample_rate, out_sample_rate))
        self.assertEqual(len(orig_data), len(orig_data),
                         msg="Filtering changed the number of channels. Was {0}, output {1}".format(len(orig_data),
                                                                                                    len(out_data)))
        self.assertEqual(orig_data[0].size, out_data[0].size,
                         msg="Output data size doesn't match the original. Was {0}, output {1}.".format(
                             orig_data[0].size, out_data[0].size))
        return out_data

    @_test_file_manager(DIRECTORY)
    def test_same_sample_rate_no_change(self, filename):
        """
        Tests that if we set the output sample rate to < 0, the sampling doesn't change, but the file is filtered.
        """
        data_filename = tf.get_abs_path('chimera_1event.log')

        # Open a reader and read the original sample rate
        orig_reader = get_reader_from_filename(data_filename)
        orig_sample_rate = orig_reader.get_sample_rate()
        orig_data = orig_reader.get_all_data()
        orig_reader.close()

        for sample_rate in (-1, 0., orig_sample_rate, orig_sample_rate + 100.):
            # filter the file, with negative sample rate
            out_filename = filter_file(data_filename, 10.e4, sample_rate, output_filename=filename)

            self._test_out_sample_rate_data_len_equality(orig_data, orig_sample_rate, out_filename, sample_rate)

            os.remove(out_filename)

    @_test_file_manager(DIRECTORY)
    def test_set_output_sample_rate(self, filename):
        """
        Tests that we can successfully set the output sample rate, and the number of data points changes correctly.
        """
        data_file_names = [tf.get_abs_path('chimera_1event.log'),
                           tf.get_abs_path('chimera_1event_2levels.log')]

        for data_filename in data_file_names:
            # Open a reader and read the original sample rate
            orig_reader = get_reader_from_filename(data_filename)
            orig_sample_rate = orig_reader.get_sample_rate()
            orig_data = orig_reader.get_all_data()
            n_orig = orig_data[0].size
            orig_reader.close()

            for out_sample_rate in (100.e4, 1.e6, 5.e5):
                out_filename = filter_file(data_filename, 10.e4, out_sample_rate, output_filename=filename)

                # The output number of data points should be int(np.ceil(n_orig * out_sample_rate / orig_sample_rate))
                n_out = int(np.ceil(n_orig * out_sample_rate / orig_sample_rate))
                # The output sample rate should be set by n_out
                out_sample_rate = orig_sample_rate * (1.0 * n_out) / n_orig

                # Get the params from the output file
                out_reader = get_reader_from_filename(out_filename)
                o_s_r = out_reader.get_sample_rate()
                out_data = out_reader.get_all_data()
                n_o = out_data[0].size
                out_reader.close()

                self.assertEqual(n_out, n_o,
                                 "Number of re-sample points not correct. "
                                 "Original data {0}, output {1}, should be {2}.".format(n_orig, n_o, n_out))
                self.assertAlmostEqual(o_s_r, out_sample_rate, 2,
                                       "Sample rate not set correctly. Was {0}, should be {1}".format(o_s_r,
                                                                                                      out_sample_rate))

                os.remove(out_filename)

    @_test_file_manager(DIRECTORY)
    def test_filtered_baseline(self, filename):
        """
        Tests that the filtered baseline is the same as the unfiltered.
        """
        data_filename = tf.get_abs_path('chimera_1event.log')

        reader = get_reader_from_filename(data_filename)
        data_all = reader.get_all_data()
        data = data_all[0]
        reader.close()

        baseline = np.mean(data[:150])

        # Check at different filter frequencies and re-sample rates
        for rates in ([1.e4, 1.e6], [1.e4, 0], [5.e5, 4.e6], [7.7e4, 1.e6]):
            filter_freq = rates[0]
            re_sample_rate = rates[1]

            out_filename = filter_file(data_filename, filter_frequency=filter_freq, out_sample_rate=re_sample_rate,
                                       output_filename=filename)

            reader = get_reader_from_filename(out_filename)
            data2 = reader.get_all_data()[0]
            reader.close()

            # Note we re-sampled, which is why only take 30 data points.
            baseline2 = np.mean(data2[:20])

            ratio = abs((baseline - baseline2) / baseline)
            print "ratio:", ratio
            self.assertLessEqual(ratio, 0.05, "Filtered baseline different from original. "
                                              "Should be {0}, got {1}.".format(baseline, baseline2))

            os.remove(out_filename)


"""
Tests for eventFinder.pyx

@author: `@parkin`_
"""
import unittest
from pypore.event_finder import find_events, get_reader_from_filename
from pypore.event_finder import _get_data_range_test_wrapper
import numpy as np
import os
import pypore.filetypes.event_database as ed

import pypore.sampledata.testing_files as tf
from pypore.tests.util import _test_file_manager


class TestEventFinder(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_get_data_range_test_wrapper_at_new(self):
        n = 100
        first = np.zeros(n)
        first[n - 1] += 1.
        first[0] += 1.
        data_cache = [first, np.zeros(n) + 100., np.zeros(n) + 100. + 60., np.zeros(n) + 200.]

        res = _get_data_range_test_wrapper(data_cache, 0, n)
        self.assertEqual(res.size, n)
        np.testing.assert_array_equal(res, np.zeros(n) + 100.)

        # Test negative i to 0
        x = np.zeros(10)
        x[9] += 1.
        res = _get_data_range_test_wrapper(data_cache, -10, 0)
        np.testing.assert_array_equal(res, x)

        # Test negative i,n
        x = np.zeros(10)
        x[0] += 1.
        res = _get_data_range_test_wrapper(data_cache, -100, -90)
        np.testing.assert_array_equal(res, x)

        # Test negative i, pos n in first spot
        x = np.zeros(10)
        x[4] += 1.
        x[5:10] += 100.
        res = _get_data_range_test_wrapper(data_cache, -5, 5)
        np.testing.assert_array_equal(res, x)

        # Test pos i,n both in first spot
        x = np.zeros(10) + 100.
        res = _get_data_range_test_wrapper(data_cache, 60, 70)
        np.testing.assert_array_equal(res, x)

        # Test first and second cache overlap
        x = np.zeros(10) + 100.
        x[5:] += 60.
        res = _get_data_range_test_wrapper(data_cache, 95, 105)
        np.testing.assert_array_equal(res, x)

        # Test fist cache bumping up on second
        x = np.zeros(10) + 100.
        res = _get_data_range_test_wrapper(data_cache, 90, 100)
        np.testing.assert_array_equal(res, x)

        # Test second cache
        x = np.zeros(10) + 160.
        res = _get_data_range_test_wrapper(data_cache, 155, 165)
        np.testing.assert_array_equal(res, x)

        # Test neg overlap with 2 pos caches
        x = np.zeros(120)
        x[9] += 1.
        x[10:] += 100.
        x[110:] += 60.
        res = _get_data_range_test_wrapper(data_cache, -10, 110)

        # Test neg overlap with 3 pos caches
        x = np.zeros(220)
        x[9] += 1.
        x[10:] += 100.
        x[110:] += 60.
        x[210:] += 40.
        res = _get_data_range_test_wrapper(data_cache, -10, 210)

    def test_saving_files(self):
        filename = tf.get_abs_path('chimera_1event.log')

        event_database = find_events([filename], save_file_names=['_testSavingFiles_9238.h5'])[0]

        self.assertTrue(os.path.isfile(event_database))

        h5file = ed.open_file(event_database, mode='r')

        self.assertTrue(h5file.isopen)

        h5file.close()

        # delete the newly created event file
        os.remove(event_database)

    def test_chimera_no_noise_1event(self):
        filename = tf.get_abs_path('chimera_nonoise_1event.log')
        event_database = find_events([filename], save_file_names=['_testChimera_nonoise_1Event_9238.h5'])[0]

        h5file = ed.open_file(event_database, mode='r')

        events = h5file.root.events

        # check event table correct length        
        event_table = events.eventTable
        self.assertTrue(event_table.nrows, 1)

        # check raw data array correct length
        raw_data_matrix = events.raw_data
        self.assertEqual(raw_data_matrix.nrows, 1)
        event_length = event_table[0]['event_length']
        self.assertEqual(event_length, 1000)

        # Make sure only 1 event with 1 level
        levels_matrix = events.levels
        self.assertEqual(levels_matrix.nrows, 1)
        n_levels = event_table[0]['n_levels']
        self.assertEqual(n_levels, 1)
        levels = levels_matrix[0]
        self.assertAlmostEqual(levels[0], 15.13955, 3)

        # Check 1 event with 1 levelLength
        lengths_matrix = events.level_lengths
        self.assertEqual(lengths_matrix.nrows, 1)
        lengths = lengths_matrix[0]
        self.assertEqual(lengths[0], 1000)

        h5file.close()

        # delete the newly created event file
        os.remove(event_database)

    def _test_chimera_no_noise_1event_2levels_helper(self, h5file):
        events = h5file.root.events

        event_table = events.eventTable
        self.assertTrue(event_table.nrows, 1)

        # check raw data array correct length
        raw_data_matrix = events.raw_data
        self.assertEqual(raw_data_matrix.nrows, 1)
        event_length = event_table[0]['event_length']
        self.assertEqual(event_length, 1500)

        levels_matrix = events.levels
        self.assertEqual(levels_matrix.nrows, 1)
        levels = levels_matrix[0]
        n_levels = event_table[0]['n_levels']
        self.assertEqual(n_levels, 2)
        self.assertAlmostEqual(levels[0], 15.13955, 3)
        self.assertAlmostEqual(levels[1], 12.70204, 3)

        levels_matrix = events.level_lengths
        self.assertEqual(levels_matrix.nrows, 1)
        levels = h5file.get_level_lengths_at(0)
        self.assertEqual(len(levels), 2)
        self.assertEqual(levels[0], 750)
        self.assertEqual(levels[1], 750)

    def test_chimera_no_noise_1event_2levels(self):
        filename = tf.get_abs_path('chimera_nonoise_1event_2levels.log')
        event_database = find_events([filename], save_file_names=['_testChimera_nonoise_1Event_2Levels_9238.h5'])[0]

        h5file = ed.open_file(event_database, mode='r')
        self._test_chimera_no_noise_1event_2levels_helper(h5file)
        h5file.close()

        # delete the newly created event file
        os.remove(event_database)

    def _test_chimera_no_noise_2events_1levels_wrapper(self, h5file):
        events = h5file.root.events

        # check event table correct length        
        event_table = events.eventTable
        self.assertTrue(event_table.nrows, 2)

        # check raw data array correct length
        raw_data_matrix = events.raw_data
        self.assertEqual(raw_data_matrix.nrows, 2)
        event_length = event_table[0]['event_length']
        self.assertEqual(event_length, 1000)
        # second event
        event_length = event_table[1]['event_length']
        self.assertEqual(event_length, 1000)

        # Make sure only 2 events with 1 level each
        levels_matrix = events.levels
        self.assertEqual(levels_matrix.nrows, 2)
        levels = levels_matrix[0]
        n_levels = event_table[0]['n_levels']
        self.assertEqual(n_levels, 1)
        self.assertAlmostEqual(levels[0], 15.1395, 2)
        # event 2
        levels = levels_matrix[1]
        n_levels_1 = event_table[1]['n_levels']
        self.assertEqual(n_levels_1, 1)
        self.assertAlmostEqual(levels[0], 15.1395, 2)

        # Check 2 events with 1 level -> 1 lengths
        lengths_matrix = events.level_lengths
        self.assertEqual(lengths_matrix.nrows, 2)
        lengths1 = h5file.get_level_lengths_at(0)
        self.assertEqual(len(lengths1), 1)
        self.assertEqual(lengths1[0], 1000)
        # event
        lengths2 = h5file.get_level_lengths_at(1)
        self.assertEqual(len(lengths2), 1)
        self.assertEqual(lengths2[0], 1000)

    def test_chimera_no_noise_2events_1levels(self):
        filename = tf.get_abs_path('chimera_nonoise_2events_1levels.log')
        event_databases = find_events([filename], save_file_names=['_testChimera_nonoise_2events_1levels_9238.h5'])

        self.assertEqual(len(event_databases), 1)

        event_database = event_databases[0]

        h5file = ed.open_file(event_database, mode='r')
        self._test_chimera_no_noise_2events_1levels_wrapper(h5file)
        h5file.close()

        # delete the newly created event file
        os.remove(event_database)

    def test_multiple_files(self):
        filename1 = tf.get_abs_path('chimera_nonoise_2events_1levels.log')
        filename2 = tf.get_abs_path('chimera_nonoise_1event_2levels.log')
        file_names = [filename1, filename2]
        event_databases = find_events(file_names,
                                      save_file_names=['_testMultipleFiles_1_9238.h5', '_testMultipleFiles_2_9238.h5'])

        self.assertEqual(len(event_databases), 2)

        h5file = ed.open_file(event_databases[0], mode='r')
        self._test_chimera_no_noise_2events_1levels_wrapper(h5file)
        h5file.close()
        os.remove(event_databases[0])

        h5file = ed.open_file(event_databases[1], mode='r')
        self._test_chimera_no_noise_1event_2levels_helper(h5file)
        h5file.close()
        os.remove(event_databases[1])

    def test_passing_reader(self):
        """
        Tests that passing an open subtype of :py:class:`pypore.i_o.abstract_reader.AbstractReader` works.
        """
        filename = os.path.dirname(os.path.realpath(__file__))
        filename = tf.get_abs_path('chimera_nonoise_2events_1levels.log')

        reader = get_reader_from_filename(filename)
        data = [reader]
        event_databases = find_events(data,
                                      save_file_names=['_test_passing_reader.h5'])

        self.assertEqual(len(event_databases), 1)

        h5file = ed.open_file(event_databases[0], mode='r')
        self._test_chimera_no_noise_2events_1levels_wrapper(h5file)
        h5file.close()
        os.remove(event_databases[0])

    def test_debug_option(self):
        filename = os.path.dirname(os.path.realpath(__file__))
        filename = tf.get_abs_path('chimera_nonoise_2events_1levels.log')

        output_filename = '_test_debug_option.h5'

        reader = get_reader_from_filename(filename)
        data = [reader]
        event_databases = find_events(data,
                                      save_file_names=[output_filename], debug=True)

        data = reader.get_all_data()[0]

        reader.close()

        self.assertEqual(len(event_databases), 1)

        h5file = ed.open_file(event_databases[0], mode='r')
        self._test_chimera_no_noise_2events_1levels_wrapper(h5file)

        # check that the file has the correct groups
        groups = [x._v_name for x in h5file.walkGroups()]
        self.assertIn('debug', groups, "No debug group in debug file.")

        print "data:", h5file.root.debug.data[:]
        np.testing.assert_array_equal(data, h5file.root.debug.data[0][:])

        h5file.close()

        os.remove(event_databases[0])


DIRECTORY = os.path.dirname(os.path.abspath(__file__))

from pypore.event_finder import Parameters
from pypore.strategies.absolute_change_threshold_strategy import AbsoluteChangeThresholdStrategy


class TestEventFinderAbsoluteChangeThresholdStrategy(unittest.TestCase):
    @_test_file_manager(DIRECTORY)
    def test_too_large_start_threshold(self, filename):
        """
        Tests that we don't find events when the starting threshold is too large.
        """
        data_file = tf.get_abs_path('chimera_1event.log')

        parameters = Parameters(threshold_strategy=AbsoluteChangeThresholdStrategy(100., 1.))

        event_databases = find_events([data_file], parameters=parameters,
                                      save_file_names=[filename], debug=True)

        h5file = ed.open_file(filename, mode='r')

        event_count = h5file.get_event_count()
        self.assertEqual(event_count, 0, "Unexpected event count. Should be {0}, was {1}.".format(event_count, 0))

        h5file.close()

    @_test_file_manager(DIRECTORY)
    def test_good_thresholds(self, filename):
        """
        Tests that we find the correct number of events when the starting and ending thresholds are appropriate.
        """
        data_file = tf.get_abs_path('chimera_1event.log')

        parameters = Parameters(threshold_strategy=AbsoluteChangeThresholdStrategy(2., 1.))

        event_databases = find_events([data_file], parameters=parameters,
                                      save_file_names=[filename], debug=True)

        h5file = ed.open_file(filename, mode='r')

        #Check the number of events
        event_count = h5file.get_event_count()
        self.assertEqual(event_count, 1, "Unexpected event count. Should be {0}, was {1}.".format(event_count, 0))

        #Check the event length
        sample_rate = h5file.get_sample_rate()
        event_length = h5file.get_event_row(0)['event_length']/sample_rate
        event_length_should_be = 0.00024
        percent_diff = abs(event_length - event_length_should_be) / event_length_should_be
        self.assertLessEqual(percent_diff, 0.05,
                             "Unexpected event length. Should be {0}, was {1}.".format(event_length_should_be,
                                                                                       event_length))

        h5file.close()

    @_test_file_manager(DIRECTORY)
    def test_too_large_end_threshold(self, filename):
        """
        Tests that we don't find events when the ending threshold is too large.
        """
        data_file = tf.get_abs_path('chimera_1event.log')

        parameters = Parameters(threshold_strategy=AbsoluteChangeThresholdStrategy(2., 1000.))

        event_databases = find_events([data_file], parameters=parameters,
                                      save_file_names=[filename], debug=True)

        h5file = ed.open_file(filename, mode='r')

        event_count = h5file.get_event_count()
        self.assertEqual(event_count, 0, "Unexpected event count. Should be {0}, was {1}.".format(event_count, 0))

        h5file.close()


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



"""
@author: `@parkin`_
"""
import os.path

from PySide import QtGui


class FileListItem(QtGui.QListWidgetItem):
    """
    Subclass of QListWidgetItem to handle file names with long file paths.
    """
    
    def __init__(self, filename):
        words = os.path.split(filename)
        self.simple_name = words[1]
        super(FileListItem, self).__init__(self.simple_name)
        self.file_name = filename
        self.directory = words[0]
        
    def get_file_name(self):
        """
        :returns: StringType -- the file names with the file path included.
                    ex /home/user/.../filename
        """
        return self.file_name
    
    def get_simple_name(self):
        """
        :returns: StringType -- the file name without the file path included.
        """
        return self.simple_name
    
    def get_directory(self):
        """
        :returns: StringType -- the directory containing the file.
        """
        return self.directory

    
class DataFileListItem(FileListItem):
    """
    Is a FileListItem with a parameter.
    """
    
    def __init__(self, filename, params):
        FileListItem.__init__(self, filename)
        self.params = params
        
    def get_params(self):
        """
        :returns: DictType -- Dictionary of the item's parameters.
        """
        return dict(self.params)
    
    def get_param(self, param):
        """
        :param param: Key in the parameter dictionary, whose value should be returned.
        :returns: Parameter value specified by the key 'param'.
        """
        if param in self.params:
            return self.params[param]
        else:
            return None


class FilterListItem(QtGui.QListWidgetItem):
    """
    Subclass of QListWidgetItem to handle filter list items.
    
    self.params contain the filter parameters specified by the user.
    """
    def __init__(self, file_names, **params):
        """
        :param ListType<String> file_names: File names that define this FilterListItem.
        :param params: A dictionary of the parameters for this FilterListItem.
        """
        self.file_names = file_names
        self.simple_names = []
        item_text = 'item'
        if len(file_names) > 0:
            item_text = ''
        index = 0
        for filename in file_names:
            words = os.path.split(filename)
            simple_name = words[1]
            item_text += simple_name
            self.simple_names.append(simple_name)
            index += 1
            # don't append ', ' to last item
            if index < len(file_names):
                item_text += ', '
        super(FilterListItem, self).__init__(item_text)
        self.params = params
        if not 'color' in params:
            # give the item a default color
            self.params['color'] = QtGui.QColor.fromRgbF(0., 0., 1.)
        # set the icon color
#         self.setForeground(params['color'])
        pix_map = QtGui.QPixmap(20, 20)
        pix_map.fill(self.params['color'])
        self.setIcon(QtGui.QIcon(pix_map))
        
    def get_params(self):
        """
        :returns: Dict -- dictionary of the parameters of this Item.
        """
        return dict(self.params)
    
    def get_file_names(self):
        """
        :returns: List -- list of the full file names of the files that make up this Item.
        """
        return list(self.file_names)
    
    def get_file_name_at(self, index):
        """
        :param IntType index: Position of the item.
        :returns: String -- Full filename (including path) of the file at index.
        """
        if 0 <= index < len(self.file_names):
            return self.file_names[index]
        else:
            return None
    
    def get_simple_names(self):
        """
        :returns: List -- list of the simple names (no path/directory) of the files that make up this Item.
        """
        return list(self.simple_names)
    
    def get_simple_name_at(self, index):
        """
        :param IntType index: Index of the list item to get the simple name of.
        :returns: String -- Simple name (i.e. no path/directory) of the file at index.
        """
        if 0 <= index < len(self.file_names):
            return self.simple_names[index]
        else:
            return None



#!/usr/bin/env python

"""
Created on Jul 23, 2013

@author: `@parkin`_
"""
import os

from PySide import QtCore
import time
from multiprocessing import Process, Pipe
from pypore.event_finder import find_events
from pypore.i_o import get_reader_from_filename


class PlotThread(QtCore.QThread):
    dataReady = QtCore.Signal(object)

    cancelled = False

    def __init__(self, axes, data='', plot_range='all', filename='', decimate=False, sample_rate=0.0):
        QtCore.QThread.__init__(self)
        self.plot_options = {'axes': axes, 'data': data, 'plot_range': plot_range}
        self.filename = filename
        self.decimate = decimate
        self.sample_rate = sample_rate

    def __del__(self):
        """
        If the object instantiating this thread gets deleted, the thread will be deleted, causing
        baseline_filter_parameter segfault, unless we implement this destructor.
        """
        self.wait()

    def run(self):
        if not self.filename == '' or self.plot_options['datadict'] == '':
            reader = get_reader_from_filename(self.filename)
            self.plot_options['data'] = reader.get_all_data(self.decimate)
            if self.sample_rate == 0.0:
                self.sample_rate = reader.get_sample_rate()
            reader.close()
        if self.cancelled:
            return
        self.plot_options['sample_rate'] = self.sample_rate
        self.dataReady.emit({'plot_options': self.plot_options, 'status_text': '', 'thread': self})


class AnalyzeDataThread(QtCore.QThread):
    """
    Class for searching for events in baseline_filter_parameter separate thread.
    """
    dataReady = QtCore.Signal(object)

    cancelled = False

    readyForEvents = True

    p = None

    def __init__(self, file_names, parameters, debug, save_file_names=None):
        QtCore.QThread.__init__(self)
        self.parameters = parameters
        self.debug = debug

        self.file_names = file_names
        self.save_file_names = save_file_names

        self.event_count = 0
        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.periodic_call)
        self.timer.setSingleShot(True)

        self.next_event_to_send = 0

        self.events = []
        self.status_text = ''

        self.periodic_call()

    def __del__(self):
        """
        If the object instantiating this thread gets deleted, the thread will be deleted, causing
        baseline_filter_parameter segfault, unless we implement this destructor.
        """
        self.wait()

    def periodic_call(self):
        self.update_gui()
        if self.cancelled:
            self.dataReady.emit({'done': True})
            if self.p is not None:
                self.p.terminate()
                self.p.join()
            return
        self.timer.start(500)

    def update_gui(self):
        #         self.dataReady.emit({'event': event})
        do_send = False
        send = {}
        if len(self.status_text) > 0:
            send['status_text'] = self.status_text
            self.status_text = ''
            do_send = True
        if len(self.events) > 0:
            send['Events'] = self.events
            self.events = []
            do_send = True
        if do_send:
            self.dataReady.emit(send)

    def cancel(self):
        self.cancelled = True

    def run(self):
        self.time1 = time.time()
        if os.name == 'posix':
            self._pipe, child_conn = Pipe()
            self.p = Process(target=find_events, args=(self.file_names,),
                             kwargs={'parameters': self.parameters, 'pipe': child_conn, 'debug': self.debug,
                                     'save_file_names': self.save_file_names})
            self.p.start()
            # child_conn needs to be closed in all processes before EOFError is thrown (on Linux)
            # So close it here immediately
            child_conn.close()
            while True:
                time.sleep(0)
                try:
                    data = self._pipe.recv()
                    if 'status_text' in data:
                        self.status_text = data['status_text']
                    if 'Events' in data:
                        self.events += data['Events']
                except:
                    break
        else:
            # TODO add ability to listen for info from find_events on Windows.
            # If we are on windows, we can only fork a process if __name__ == '__main__'. Which
            # is not true here (because AnalyzeDataThread is imported).
            # So on Windows, just use this thread, don't use an additional separate process.
            find_events(self.file_names, parameters=self.parameters, debug=self.debug,
                        save_file_names=self.save_file_names)

        self.cancelled = True



import os
import unittest

from PySide import QtGui
from pypore.tests.util import _test_file_manager

from pyporegui.my_threads import AnalyzeDataThread
import pypore.sampledata.testing_files as tf
from pypore.event_finder import Parameters

# This test class needs a QApplication instance
if QtGui.qApp is None:
    QtGui.QApplication([])

DIRECTORY = os.path.dirname(os.path.abspath(__file__))


class TestAnalyzeDataThread(unittest.TestCase):
    @_test_file_manager(DIRECTORY)
    def test_one_file_with_events(self, filename):
        """
        Tests that events are found and event databases are saved.
        """
        test_data_file_names = tf.get_all_file_names()

        test_file = tf.get_abs_path('chimera_1event.log')
        analyze_thread = AnalyzeDataThread([test_file], Parameters(), debug=True, save_file_names=[filename])
        analyze_thread.start()

        # wait until the thread has finished
        analyze_thread.wait()

        self.assertTrue(os.path.exists(filename))




"""
@author: `@parkin`_
"""
import unittest

from pypore.i_o.chimera_reader import ChimeraReader
from pypore.i_o.tests.reader_tests import ReaderTests
import pypore.sampledata.testing_files as tf


class TestChimeraReader(unittest.TestCase, ReaderTests):
    reader_class = ChimeraReader

    default_test_data_files = [tf.get_abs_path('chimera_small.log'),
                               tf.get_abs_path('spheres_20140114_154938_beginning.log')]

    def test_constructor_no_mat_spec(self):
        """
        Tests that an IOError is raised when no .mat spec file is next to the .log file.
        """
        test_no_mat_chimera_files = tf.get_abs_path('chimera_empty.log')
        for filename in test_no_mat_chimera_files:
            self.assertRaises(IOError, ChimeraReader, filename)

    def test_get_all_data(self):
        # Make sure path to chimera file is correct.
        filename = tf.get_abs_path('chimera_small.log')
        chimera_reader = ChimeraReader(filename)
        data = chimera_reader.get_all_data(False)
        self._test_small_chimera_file_help(data)
        chimera_reader.close()

    def _test_small_chimera_file_help(self, data_all):
        self.assertEqual(len(data_all), 1, 'Too many data channels returned.')
        data = data_all[0]
        self.assertEqual(data.size, 10, 'Wrong data size returned.')
        self.assertAlmostEqual(data[0], 17.45518, 3)
        self.assertAlmostEqual(data[9], 18.0743, 3)

    def help_scaling(self):
        filename = tf.get_abs_path('spheres_20140114_154938_beginning.log')
        mean_should_be = 7.57604  # Value gotten from original MATLAB script
        std_should_be = 1.15445  # Value gotten from original MATLAB script
        return [filename], [mean_should_be], [std_should_be]

    def help_scaling_decimated(self):
        filename = tf.get_abs_path('spheres_20140114_154938_beginning.log')
        return [filename]


if __name__ == "__main__":
    unittest.main()




"""
Created on Jan 28, 2014

@author: `@parkin`_
"""
import os
import datetime

from filetypes import data_file
from pypore.i_o import get_reader_from_filename
import pypore.filetypes.data_file as df
from pypore.i_o.abstract_reader import AbstractReader


def convert_file(filename, output_filename=None):
    """
    Convert a file to the pypore .h5 file format. Returns the new file's name.
    """
    reader = get_reader_from_filename(filename)

    sample_rate = reader.get_sample_rate()
    n_points = reader.get_points_per_channel_total()

    if output_filename is None:
        output_filename = filename.split('.')[0] + '.h5'

    save_file = data_file.open_file(output_filename, mode='w', sample_rate=sample_rate, n_points=n_points)
    blocks_to_get = 1
    data = reader.get_next_blocks(blocks_to_get)[0]

    n = data.size
    i = 0
    while n > 0:
        save_file.root.data[i:n + i] = data[:]
        i += n
        data = reader.get_next_blocks(blocks_to_get)[0]
        n = data.size

    reader.close()

    save_file.flush()
    save_file.close()
    return output_filename


class SamplingRatesMismatchError(Exception):
    pass


def concat_files(files, output_filename=None):
    """
    This function concatenates multiple files into one data file. All of the sampling rates of the original files
    must be the same.

    :param list files: List of string file names OR
        :py:class:`Readers <pypore.i_o.abstract_reader.AbstractReader>`.
    :param output_filename: Optional file name for the resulting file.
    :raises: :py:exc:`ValueError` -- if the length of the files list is < 2.
    :raises: :py:exc:`SamplingRatesMismatchError <pypore.file_converter.SamplingRatesMismatchError>` -- if the sampling
        rates do not match in all of the files.

    >>> from pypore.i_o.data_file_reader import DataFileReader
    >>> concat_files(['file1.log', DataFileReader('dataFile.h5')], output_filename='concatenated.h5') # can pass strings or Readers
    """
    if len(files) < 2:
        raise ValueError("Minimum length of files list is 2.")

    # Get the first sample rate
    should_close_reader = False
    reader = files[0]
    if not isinstance(reader, AbstractReader):
        reader = get_reader_from_filename(reader)
        should_close_reader = True

    sample_rate = reader.get_sample_rate()

    if output_filename is None:
        basename = os.path.basename(reader.get_filename())
        output_filename = basename.split('.')[0] + '_concatenated_' + datetime.datetime.now().strftime(
            "%Y%m%d_%H%M%S") + '.h5'

    if should_close_reader:
        reader.close()

    n = 0

    # Get the total number of data points, and check that the sampling rates are equal.
    for i, reader in enumerate(files):
        should_close_reader = False

        # If it's not already a reader
        if not isinstance(reader, AbstractReader):
            reader = get_reader_from_filename(reader)
            should_close_reader = True

        curr_sample_rate = reader.get_sample_rate()

        if curr_sample_rate != sample_rate:
            raise SamplingRatesMismatchError(
                "Sampling rates differ in files. Found {0} and {1}.".format(curr_sample_rate, sample_rate))

        n += reader.get_all_data()[0].size

        if should_close_reader:
            reader.close()

    # Open a new data file.
    new_data_file = df.open_file(output_filename, mode='w', n_points=n, sample_rate=sample_rate)

    curr_i = 0

    for i, reader in enumerate(files):
        should_close_reader = False

        # If it's not already a reader
        if not isinstance(reader, AbstractReader):
            reader = get_reader_from_filename(reader)
            should_close_reader = True

        n_i = reader.get_points_per_channel_total()

        new_data_file.root.data[curr_i:curr_i + n_i] = reader.get_all_data()[0]

        curr_i += n_i

        if should_close_reader:
            reader.close()

    new_data_file.close()

    return output_filename




"""
Created on Jan 28, 2014

@author: `@parkin`_
"""

import tables as tb


class DataFile(tb.file.File):
    """
    PyTables HDF5 database storing raw data.
    Inherits from tables.file.File, so you can interact with this
    just as you would a PyTables File object. However, this contains
    some extra convenience methods for storing/reading data. Note that 
    DataFile allows you to interact
    with this PyTables file in the usual PyTables file manner, so you
    can potentially mangle the data from the original DataFile
    format.
    
    Automatically adds matrix
    /data
    
    Must be instantiated by calling :py:func:`pypore.filetypes.data_file.open_file`
    
    >>> import pypore.dataFile as df
    >>> database = df.open_file(tests,mode='w')
    >>> # Now do stuff with the DataFile
    >>> # ...
    >>> # Close and remove the DataFile
    >>> database.close()
    >>> os.remove(tests)
    """

    def clean_database(self):
        """
        Removes /events and then re-initializes the /events group. Note
        that any references to any table/matrix in this group will
        be broken and need to be refreshed.
        
        >>> h5 = open_file(tests,mode='a')
        >>> table = h5.get_event_table()
        >>> h5.clean_database() // table is now refers to deleted table
        >>> table = h5.get_event_table() // table now refers to live table
        """
        # remove the events group
        self.root._f_remove(recursive=True)

        self.initialize_database()

    @classmethod
    def _convert_to_data_file(cls, tables_object):
        """
        Converts a PyTables object's __class__ field to DataFile so
        you can use the object as an DataFile object.
        """
        tables_object.__class__ = DataFile

    def get_data_length(self):
        """
        Note this will flush the table so the data is correct.
        :returns: The number of rows in the data matrix.
        """
        self.root.data.flush()
        return self.root.data.nrows

    def get_sample_rate(self):
        """
        :returns: The sample rate at root.attrs.sample_rate
        """
        return self.root.data.attrs.sample_rate

    def initialize_database(self, **kargs):
        """
        Initializes the data_file.

        :param kargs: Can pass in 'n_points': Maximum number of data points for an event to be added.
        """

        filters = tb.Filters(complib='zlib', complevel=3)
        shape = (kargs['n_points'],)
        a = tb.FloatAtom()
        if not 'data' in self.root:
            self.createCArray(self.root, 'data', a, shape=shape, title='Data', filters=filters)

        # set the attributes
        self.root.data.attrs.sample_rate = kargs['sample_rate']


def open_file(*args, **kargs):
    """
    Opens a :py:class:`DataFile`, which is a subclass of :py:class:`tables.file.File` and can be treated as such.

    :param args: Arguments that get passed to :py:func:`tables.openFile`.
    :param kargs: Arguments that get passed to :py:func:`tables.openFile`. Should additionally include:

        - n_points: Number of points that should be in the array.
        - sample_rate: Sample rate of the data.

    :returns: :py:class:`pypore.filetypes.data_file.DataFile` -- an already opened
        :py:class:`pypore.filetypes.data_file.DataFile`.

    >>> import pypore.filetypes.data_file as df
    >>> df.open_file('tests.h5', mode='w')
    """
    f = tb.openFile(*args, **kargs)
    DataFile._convert_to_data_file(f)
    if 'mode' in kargs:
        mode = kargs['mode']
        if mode in ('w', 'a'):
            f.initialize_database(**kargs)
    return f



"""
Created on Sep 13, 2013

@author: `@parkin`_
"""

import tables as tb
import csv


# Description of events table
class _Event(tb.IsDescription):
    """
    Description of the table /events/eventTable.
    """
    # UIntAtom = uint32
    array_row = tb.UIntCol(pos=0)  # indicates the corresponding row in the
    event_start = tb.UIntCol(itemsize=8, pos=1)  # start index of the event in the data
    event_length = tb.UIntCol(pos=2)
    n_levels = tb.UIntCol(pos=3)
    raw_points_per_side = tb.UIntCol(pos=4)
    baseline = tb.FloatCol(pos=5)
    current_blockage = tb.FloatCol(pos=6)
    area = tb.FloatCol(pos=7)


class EventDatabase(tb.file.File):
    """
    PyTables HDF5 database storing events and corresponding data.
    Inherits from tables.file.File, so you can interact with this
    just as you would a PyTables File object. However, this contains
    some extra convenience methods for storing/reading events
    and event data. Note that EventDatabase allows you to interact
    with this PyTables file in the usual PyTables file manner, so you
    can potentially mangle the data from the original EventDatabase
    format.
    
    Automatically adds a group
    /events
    With table
    /events/eventTable
    and matrices
    /events/raw_data, /event/levels, and /event/levelLength
    
    Must be instantiated by calling eventDatabase's
    
    >>> import pypore.eventDatabase as ed
    >>> database = ed.open_file('tests.h5',mode='w')
    >>> database.close()
    >>> os.remove('tests.h5')
    """

    DEFAULT_MAX_EVENT_LENGTH = 100
    max_event_length = DEFAULT_MAX_EVENT_LENGTH
    event_row = None

    def append_event(self, array_row, event_start, event_length, n_levels, raw_points_per_side, baseline, current_blockage, area,
                    raw_data=None, levels=None, level_lengths=None):
        """
        Appends an event with the specified values to the eventsTable.  If raw_data, levels, or level_lengths
        are included, they are added to the corresponding matrices.
        
        :param Int array_row: The row in the raw_data, levels, and level_lengths array that corresponds
                             to this event.
        :param event_start: The starting index of this event in the data.
        :param event_length: Number of data points in the event.
        :param n_levels: Number of levels in the event.
        :param raw_points_per_side: Number of extra points kept on either side of the event in raw_data.
        :param baseline: Open-pore current at the time of the event.
        :param current_blockage: The mean current blockage of the event.
        :param area: The area of the event.
        :param raw_data: Numpy array of the raw data.
        :param levels: Numpy array of the levels.
        :param level_lengths: Numpy array of the level lengths.
        """
        row = self.get_event_table_row()
        row['array_row'] = array_row
        row['event_start'] = event_start
        row['event_length'] = event_length
        row['n_levels'] = n_levels
        row['raw_points_per_side'] = raw_points_per_side
        row['baseline'] = baseline
        row['current_blockage'] = current_blockage
        row['area'] = area
        row.append()

        if raw_data is not None:
            self.append_raw_data(raw_data)
        if levels is not None:
            self.append_levels(levels)
        if level_lengths is not None:
            self.append_level_lengths(level_lengths)

    def append_level_lengths(self, level_lengths):
        """
        Appends a numpy matrix level_lengths to root.events.level_lengths
        """
        if level_lengths is not None:
            self.root.events.level_lengths.append(level_lengths)

    def append_levels(self, levels):
        """
        Appends a numpy matrix levels to root.events.levels
        """
        if levels is not None:
            self.root.events.levels.append(levels)

    def append_raw_data(self, raw_data):
        """
        Appends a numpy matrix raw_data to root.events.raw_data
        """
        if raw_data is not None:
            self.root.events.raw_data.append(raw_data)

    def clean_database(self):
        """
        Removes /events and then re-initializes the /events group. Note
        that any references to any table/matrix in this group will
        be broken and need to be refreshed.
        
        >>> h5 = open_file('tests.h5',mode='a')
        >>> table = h5.get_event_table()
        >>> h5.clean_database() // table is now refers to deleted table
        >>> table = h5.get_event_table() // table now refers to live table
        """
        # remove the events group
        self.root.events._f_remove(recursive=True)

        self.initialize_database()

    @classmethod
    def _convert_to_event_database(cls, tables_object):
        """
        Converts a PyTables object's __class__ field to EventDatabase so
        you can use the object as an EventDatabase object.
        """
        tables_object.__class__ = EventDatabase

    def get_event_count(self):
        """
        Returns the number of rows in the /events/eventTable table.
        Note this will flush the table so the data is correct.
        """
        self.get_event_table().flush()
        return self.get_event_table().nrows

    def get_event_data_at(self, i):
        """
        Returns the event data portion of raw_data[i], ie excluding the
        raw buffer points kept on each side of an event.
        """
        row = self.get_event_row(i)
        array_row = row['array_row']
        event_length = row['event_length']
        raw_points_per_side = row['raw_points_per_side']
        return self.root.events.raw_data[array_row][raw_points_per_side:event_length + raw_points_per_side]

    def get_event_row(self, i):
        """
        Returns the ith row in /events/eventTable. Throws IndexOutOfBounds
        or similar error if i out of bounds. Note this flushes the
        eventTable before returning.
        """
        self.root.events.eventTable.flush()
        return self.root.events.eventTable[i]

    def get_events_group(self):
        """
        Returns the events group in the PyTables HDF5 file.
        """
        return self.root.events

    def get_event_table(self):
        """
        returns /events/eventTable
        """
        return self.root.events.eventTable

    def get_event_table_row(self):
        """
        Gets the PyTables Row object of the eventTable.
        root.events.eventTable.row
        
        If you need a specific row in eventTable, use getEventRow(i)
        """
        if self.event_row is None:
            self.event_row = self.root.events.eventTable.row
        return self.event_row

    def get_level_lengths_at(self, i):
        """
        Returns a numpy array of the level_lengths corresponding to the event
        in row 'i' of eventTable.
        """
        row = self.get_event_row(i)
        array_row = row['array_row']
        n_levels = row['n_levels']
        return self.root.events.level_lengths[array_row][:n_levels]

    def get_levels_at(self, i):
        """
        Returns a numpy array of the levels corresponding to the event
        in row 'i' of eventTable.
        """
        row = self.get_event_row(i)
        array_row = row['array_row']
        n_levels = row['n_levels']
        return self.root.events.levels[array_row][:n_levels]

    def get_raw_data_at(self, i):
        """
        Returns the raw_data numpy matrix associated with event 'i'.
        """
        row = self.get_event_row(i)
        array_row = row['array_row']
        event_length = row['event_length']
        raw_points_per_side = row['raw_points_per_side']
        return self.root.events.raw_data[array_row][:event_length + 2 * raw_points_per_side]

    def get_sample_rate(self):
        """
        Gets the sample rate at root.events.eventTable.attrs.sample_rate
        """
        return self.root.events.eventTable.attrs.sample_rate

    def initialize_database(self, **kargs):
        """
        Initializes the EventDatabase.  Adds a group 'events' with
        table 'eventsTable' and matrices 'raw_data', 'levels', and 'level_lengths'.

        :param kargs: Dictionary - includes:
                        -maxEventLength: Maximum number of datapoints for an event to be added.
        """
        if 'maxEventLength' in kargs:
            if kargs['maxEventLength'] > self.max_event_length:
                self.max_event_length = kargs['maxEventLength']
        if 'events' not in self.root:
            self.createGroup(self.root, 'events', 'Events')

        if not 'eventTable' in self.root.events:
            self.createTable(self.root.events, 'eventTable', _Event, 'Event parameters')
            self.event_row = None

        filters = tb.Filters(complib='zlib', complevel=3)
        shape = (0, self.max_event_length)
        a = tb.FloatAtom()
        b = tb.IntAtom()

        if not 'raw_data' in self.root.events:
            self.createEArray(self.root.events, 'raw_data',
                              a, shape=shape,
                              title="Raw data points",
                              filters=filters)

        if not 'levels' in self.root.events:
            self.createEArray(self.root.events, 'levels',
                              a, shape=shape,
                              title="Cusum levels",
                              filters=filters)

        if not 'level_lengths' in self.root.events:
            self.createEArray(self.root.events, 'level_lengths',
                              b, shape=shape,
                              title="Lengths of the cusum levels",
                              filters=filters)

        # Create/init the debug group if needed.
        if 'debug' in kargs and kargs['debug']:
            if not 'debug' in self.root:
                self.createGroup(self.root, 'debug', 'Debug')
            debug_shape = (kargs['n_channels'], kargs['n_points'])
            if not 'data' in self.root.debug:
                self.createCArray(self.root.debug, 'data',
                                  a, shape=debug_shape,
                                  title="Raw data",
                                  filters=filters)

            if not 'baseline' in self.root.debug:
                self.createCArray(self.root.debug, 'baseline',
                                  a, shape=debug_shape,
                                  title="Baseline data",
                                  filters=filters)

            if not 'threshold_positive' in self.root.debug:
                self.createCArray(self.root.debug, 'threshold_positive',
                                  a, shape=debug_shape,
                                  title="Raw data",
                                  filters=filters)

            if not 'threshold_negative' in self.root.debug:
                self.createCArray(self.root.debug, 'threshold_negative',
                                  a, shape=debug_shape,
                                  title="Raw data",
                                  filters=filters)

    def is_debug(self):
        """
        :returns: True if the event was created with the debug keyword.
        """
        return 'debug' in self.root

    def remove_event(self, i):
        """
        Deletes event i from /events/eventTable. Does nothing if
        i < 0 or i >= eventCount. Note the table will be flushed.
        Note that deleting a row in a table of length 1 is not
        currently supported.
        """
        self.remove_events(i, i + 1)

    def remove_events(self, i, j):
        """
        Deletes events [i,j) from /events/eventTable. Does nothing
        if deleting out of range events is requested. Note the table
        will be flushed. Note that deleting all the rows in a 
        table is not currently supported. Refer to cleanDatabase for
        deleting everything.
        
        Args:
            i - First entry to delete.  Must be within range
                0 < i < eventCount
            j - 1 past last entry to delete.  Must be within range
                i < j <= eventCount
        """
        event_count = self.get_event_count()

        if 0 <= i < event_count and i < j <= event_count:
            # Currently cannot delete EVERY row in a table.
            if j - i < event_count:
                self.get_event_table().removeRows(i, j)
            else:
                print "removeEvents FAILED: Removing all rows in table not currently supported."
        self.get_event_table().flush()

    def to_csv(self, filename):
        ofile = open(filename, 'wb')

        writer = csv.writer(ofile, quoting=csv.QUOTE_NONNUMERIC)

        # Write header
        header = ['Start Time', 'Dwell Time', 'Blockage', 'Baseline', 'Inter Event Time', 'Number of Levels']
        writer.writerow(header)

        table = self.get_event_table()

        sample_rate = self.get_sample_rate()

        prev_event_start = 0.
        for row in table.iterrows():
            r = [row['event_start'] / sample_rate, row['event_length'] / sample_rate, row['current_blockage'],
                 row['baseline'], (row['event_start'] - prev_event_start) / sample_rate, row['n_levels']]
            writer.writerow(r)
            prev_event_start = row['event_start']


def open_file(*args, **kargs):
    """
    Opens an EventDatabase by calling tables.openFile and then
    copying the __dict__ to a new EventDatabase instance.
    
    :param kargs: Pass in the following named parameters.

        - maxEventLength: Maximum length of an event for the table. Default is 100.
        - debug: boolean -- If debug, an extra root.debug group will be created. If passing debug=True, then\
            you need to also pass the following parameters. This mode is used by\
            :py:func:`pypore.event_finder.find_events`, and only does anything if you are opening a new databse.

            - n_points: number of points in the original data.
            - n_channels: number of channels in the original data.

            And optional parameters

            - threshold_positive: boolean -- True if you need an array allocated for positive threshold.
            - threshold_negative: boolean -- True if you need an array allocated to keep negative threshold data.

    """
    f = tb.openFile(*args, **kargs)
    EventDatabase._convert_to_event_database(f)
    if 'mode' in kargs:
        mode = kargs['mode']
        if 'w' in mode or 'a' in mode:
            f.initialize_database(**kargs)
    return f
